'FEM立体ﾃﾞｰﾀV96の標準ﾓｼﾞｭｰﾙ      ［作成］黒田英夫(2023/03)
Imports System.Drawing.Printing

Module SolDCom
    'ＦＥＭ用の立体構造データを作成
    '六面体２次要素使用
    'ＦＥＭ断面は四角形２次要素を使用
    '断面メッシュデータを用いて、柱状体、軸回転体の立体データを生成
    '構造を合成した立体データも生成
    '柱状体交差の合成機能有り
    '    ベースの柱状体に対して、側面に別の柱状体が交差
    '    柱状体は円柱または角柱に限定
    '要素番号、節点番号とも空き番号は不可

    'V86での改良点・・・データのファイル印刷メニューを廃止
    '                   （文字コードによる不具合のため）
    'V87での改良点・・・構造の合成が文字コードUTF-8のデータに対応しないエラー修正
    'V88での改良点・・・３次元流れ・熱伝導解析へのデータ出力を追加
    '                   （六面体２次要素、文字コードUTF-8のみ）
    'V89での改良点・・・節点付番の改良で各座標軸方向のバンド幅を算出
    '                   （これまでは最長方向の座標軸を採用）
    'V90での改良点・・・全体データの合成ができないバグを修正
    'V91での改良点・・・Netgenメッシュの読み込みを廃止し、Gmshメッシュの読み込みに変更
    'V92での改良点・・・四面体要素の廃止
    'V93での改良点・・・要素の追加でエラー発生を防止
    'V94での改良点・・・全体データのデータチェックに要素共通面の節点を追加
    'V95での改良点・・・部分自動メッシュの右端面作画でバグ訂正
    'V96での改良点・・・フリーソフト化、VB2022に移行、文字コードはUTF-8のみ採用、
    '                   Gmshの廃止

    Public Const Dpai As Single = 6.283185 '2π
    Public Const Hpai As Single = 1.570796 'π/2
    Public Const Pai As Single = 3.141593  'π

    Public Cr$              'CR+LFのコード
    Public DirOpen$         '立ち上げ時のディレクトリー名
    Public Fadd$            '連結する断面ファイル名
    Public Fadd3$           '連結する立体ファイル名
    Public Fauc$            '交差データの仕様ファイル名
    Public Faum$            '断面生成データのファイル名
    Public Faun$            '空間断面の生成データのファイル名
    Public Fauz$            '部分データの仕様ファイル名
    Public Fm$              '3D構造のデータファイル名
    Public Fmesh$           'meshファイル名
    Public Fy$              '解析用データのファイル名
    Public Title$           'タイトル

    Public InOut As Integer         'ファイル入出力の種類（0:入力、1:出力）
    Public Kaxis As Integer         '作図のＸＹ軸の入換え（0:なし、1:有り）
    Public KdataBackup As Integer   '入力データのバックアップファイル作成（0:なし、1:有り）
    Public KdataChange3 As Integer  '部分構造仕様データ変更の有無（0:なし、1:有り）
    Public KdataChange4 As Integer  '交差柱状体仕様データ変更の有無（0:なし、1:有り）
    Public KdataChange5 As Integer  '立体データ変更の有無（0:なし、1:有り）
    Public KdispENum As Integer     '要素番号の表示有無（0:なし、1:有り）
    Public KdispMat As Integer      '作画の材料の指定番号（0は全材料）
    Public KdispPNum As Integer     '節点番号の表示有無（0:なし、1:有り）
    Public KdispSNum As Integer     '面番号の表示有無（0:なし、1:有り）
    Public KE1 As Integer           '開始要素番号
    Public KE2 As Integer           '最終要素番号
    Public KeLeft As Integer        '左端面の基準要素番号
    Public KeRight As Integer       '右端面の相当要素番号
    Public KfileOK As Integer       'ﾌｧｲﾙのOK有無（0:なし、1:有り）
    Public KindBase As Integer      'ﾍﾞｰｽ構造種類（0:円柱、角柱）
    Public KindBody As Integer      '構造体の種類（0:柱状体、1:回転体）
    Public KindConnect As Integer   '両端面の接続方法（0:直線、１：曲線）
    Public KindElem As Integer      '要素種類（0:六面体20節点、1:四面体10節点）
    '    Public KindEncod As Integer     'ファイルの文字コード（0:UTF-8、1:Shif-JIS）
    Public Kindex As Integer        'ﾃｷｽﾄｱﾚｲのｲﾝﾃﾞｯｸｽ
    Public KindSect As Integer      '部分の両端断面種類（0:同一、1:相似、2:異形）
    Public KmouseElem As Integer    'ﾏｳｽ指定の要素番号
    Public KmouseSurf As Integer    'ﾏｳｽ指定の面番号
    Public KnumDisp As Integer      '構造作画での番号表示（0:なし、1:有り）
    Public KnumSb As Integer        'ﾌｫﾝﾄ文字ｻｲｽﾞの操作変数
    Public KnumSize As Integer      '構造作画での番号ｻｲｽﾞ
    Public KpLeft As Integer        '左端面の基準節点番号
    Public KpRight As Integer       '右端面の相当節点番号
    Public KrearTreat As Integer    '隠面処理の有無（0:なし、1:有り）
    Public Kscreen As Integer       '作画画面サイズ（0：全画面、1：通常画面）
    Public Ndel As Integer          '削除要素数
    Public NEcross As Integer       '交差面の要素数
    Public Nelem As Integer         '要素数
    Public Ngaus As Integer         'ガウス積分点数(=3)
    Public Nmat As Integer          '材料データ数
    Public Npoint As Integer        '節点数
    Public NSdraw As Integer        '作画面の数（隠面処理時）
    Public Nsect As Integer         '部分端面間のメッシュ数

    Public AngleMesh As Single      '柱状体の捩れ角、または交差角度
    Public AngleMesh2 As Single     '終了角度
    Public Cfix As Single           'XY方向ﾋﾟｸｾﾙ差の補正係数
    Public DistFactor As Single     '視点とモデルとの距離倍率（標準1）
    Public EyeAngle As Single       '視点と原点を結ぶ線の水平面との角度
    Public Eyeh As Single           '視点高さ
    Public HFactor As Single        '視点の高さ倍率（基準2）
    Public RatioSim As Single       '右端面の左端面に対する相似倍率
    Public Rlenmax As Single        'ﾓﾃﾞﾙの1軸の最大長さ
    Public Rlenmodel As Single      '視点の距離（座標系間）
    Public Vbase As Single          'ﾍﾞｰｽ構造の半径またはY座標
    Public XaxisMax As Single       'Ｘ軸の最大値
    Public XaxisMin As Single       'Ｘ軸の最小値
    Public XBL, YBL As Single       '左端面の基準位置座標
    Public XBR, YBR As Single       '右端面の基準位置座標
    Public Xlower As Single         '作画領域のX最小値
    Public XRotate As Single        'モデルのX軸回りの回転角（右ねじ正、°）
    Public Xupper As Single         '作画領域のX最大値
    Public XYscale As Single        '作画スケール
    Public YaxisMax As Single       'Ｙ軸の最大値
    Public YaxisMin As Single       'Ｙ軸の最小値
    Public Ylower As Single         '作画領域のY最小値
    Public YRotate As Single        'モデルのY軸回りの回転角（右ねじ正、°）
    Public Yupper As Single         '作画領域のY最大値
    Public ZRotate As Single        'モデルのZ軸回りの回転角（右ねじ正、°）

    Public ArrText As New ArrayList()

    Public UFile(5, 6) As String    '最近使用したﾌｧｲﾙの保存配列

    Public Kcomp() As Integer       '元ﾃﾞｰﾀ節点の計算変数順序
    Public Kdel() As Integer        '削除要素番号
    Public KdrawElem() As Integer   '各要素の作画有無（0:なし、1:有り）
    Public KEorder() As Integer     '作画順の要素番号
    Public KinnerLine() As Integer  '内部辺の指定（0:なし、1:有り）
    Public KnewUFile(5) As Integer  '最近使用ﾌｧｲﾙの更新有無
    Public KE3DCut(2) As Integer    '構造図の面切断の指定（座標軸番号と正負選択）
    Public Kprof(,) As Integer      '各線分の定義内容（指定区分と３節点番号）
    Public Krotp() As Integer       '円弧の回転方向（1:左回り、-1:右回り)
    Public KSelem(,) As Integer     '各要素面の隠面指定（0:なし、1：有り、-1:視線外）
    Public KSorder() As Integer     '作画順の要素の面番号
    Public KsurfPoint(8) As Integer '面の8節点の節点順位
    Public NNelem(,) As Integer     '各要素における全節点番号
    Public NumMat() As Integer      '各要素の材料種類番号

    Public AxisMax(3) As Single     'モデルのXYZ各軸の最大値
    Public AxisMin(3) As Single     'モデルのXYZ各軸の最小値
    Public CenterPos(3) As Single   '構造中心の座標
    Public CosLX(3), CosLY(3) As Single   '左端断面のX,Y軸の方向余弦
    Public CosRX(3), CosRY(3) As Single   '右端断面のX,Y軸の方向余弦
    Public Ds(20), Dt(20), Du(20) As Single  '形状関数のξ偏微分、η偏微分、ζ偏微分
    Public RinvJ(3, 3) As Single    '逆ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽ
    Public RJmat(3, 3) As Single    'ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽ
    Public PosGaus(3) As Single     '積分点の位置
    Public Shape(20) As Single      '形状関数
    Public Theta1(), Theta2() As Single '円弧の開始、終了角度（ﾗｼﾞｱﾝ、左回りに整理済）
    Public TransMat(4, 4) As Single  'モデルの合成アフィン変換行列
    Public VE3DCut(2) As Single     '構造図の面切断の座標値と許容誤差
    Public WeightG(3) As Single     '積分点の重み
    Public X(), Y(), Z() As Single  '節点座標
    Public Xdata(), Ydata(), Zdata() As Single    '節点座標
    Public Xdraw(8), Ydraw(8) As Single   '色塗り多角形の座標
    Public XE(20), YE(20), ZE(20) As Single   '要素計算の配列
    Public Xmodel(), Ymodel(), Zmodel() As Single 'モデル座標における節点座標
    Public Xmouse(2), Ymouse(2) As Single   'ﾏｳｽ位置座標
    Public Xscreen(), Yscreen() As Single   'スクリーンにおける節点座標
    Public Xsect(), Ysect(), Zsect() As Single  '部分断面の位置座標
    Public Xview(), Yview(), Zview() As Single  '視点座標における節点座標（左手系）

    Sub AngleCalc(ByVal xdist As Single, ByVal ydist As Single, _
        ByRef rad As Single)
        '角度計算
        'xdist･･･X長さ
        'ydist･･･Y長さ
        'rad･･･角度(ﾗｼﾞｱﾝ)
        Dim icc As Integer
        Dim hpai As Single

        hpai = 1.570796
        If xdist = 0 Then
            If ydist > 0 Then
                rad = hpai
            Else
                rad = -hpai
            End If
        ElseIf ydist = 0 Then
            If xdist > 0 Then
                rad = 0
            Else
                rad = Pai
            End If
        Else
            rad = Math.Atan(ydist / xdist)
            If xdist < 0 Then
                rad = rad + Pai
            End If
        End If

        Do
            icc = 0
            If rad < 0 Then
                icc = 1
                rad = rad + Dpai
            ElseIf rad > Dpai Then
                rad = rad - Dpai
            End If
        Loop While icc > 0

    End Sub

    Sub AucFileInputU(ByVal fxx$)
        '交差仕様ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim ns2 As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindBase = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Vbase = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        AngleMesh = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nsect = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Faum$ = dat(1)
        ns2 = 2 * Nsect
        ReDim Xsect(ns2), Ysect(ns2), Zsect(ns2)

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        XBL = dat(1)
        YBL = dat(2)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        CosLX(1) = dat(1)
        CosLX(2) = dat(2)
        CosLX(3) = dat(3)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        CosLY(1) = dat(1)
        CosLY(2) = dat(2)
        CosLY(3) = dat(3)

        aa$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Xsect(0) = dat(1)
        Ysect(0) = dat(2)
        Zsect(0) = dat(3)

        sr.Close()

    End Sub

    Sub AucFileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim a1$, aa$

        Dim sw As New IO.StreamWriter(fxx$)

        sw.WriteLine(Title$)
        aa$ = "ベース構造種類（0:円柱、1:角柱）" + "," + Str(KindBase)
        sw.WriteLine(aa$)
        If KindBase = 0 Then
            a1$ = "円柱の半径"
        Else
            a1$ = "角柱側面のＹ座標"
        End If
        aa$ = a1$ + "," + Str(Vbase)
        sw.WriteLine(aa$)
        aa$ = "交差側のＸ軸に対する交差角（°）" + "," + Str(AngleMesh)
        sw.WriteLine(aa$)
        aa$ = "交差側の両端面間のメッシュ数" + "," + Str(Nsect)
        sw.WriteLine(aa$)
        aa$ = "[交差側の断面ファイル名]" + "," + Faum$
        sw.WriteLine(aa$)

        aa$ = "左端面内の基準位置ＸB、ＹB" + "," + Str(XBL) + "," + Str(YBL)
        sw.WriteLine(aa$)
        aa$ = "左端断面Ｘ軸の方向余弦" + "," + Str(CosLX(1)) + ","
        aa$ = aa$ + Str(CosLX(2)) + "," + Str(CosLX(3))
        sw.WriteLine(aa$)
        aa$ = "左端断面Ｙ軸の方向余弦" + "," + Str(CosLY(1)) + ","
        aa$ = aa$ + Str(CosLY(2)) + "," + Str(CosLY(3))
        sw.WriteLine(aa$)

        sw.WriteLine("左端断面の位置データ（面番号、Ｘ、Ｙ、Ｚ）")
        aa$ = Str$(0) + ",  " + Str$(Xsect(0))
        aa$ = aa$ + ",  " + Str$(Ysect(0))
        aa$ = aa$ + ",  " + Str$(Zsect(0))
        sw.WriteLine(aa$)

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub AreaCalc(ByRef area As Single, ByVal x1 As Single,
    ByVal y1 As Single, ByVal x2 As Single, ByVal y2 As Single,
    ByVal xb As Single, ByVal yb As Single)
        '三角形の符号付き面積算出（ｽｸﾘｰﾝ座標で、最初の2点に対して左側は正、右側は負）
        '通常のXY座標系とは正負が逆になる
        'area･･･符号付き面積
        'x1,y1,x2,y2･･･2点の座標
        'xb,yb･･･対象点の座標
        area = -x1 * y2 + y1 * x2 - x2 * yb + y2 * xb - xb * y1 + yb * x1
    End Sub

    Sub AuzFileInputU(ByVal fxx$)
        '部分仕様ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim a4$, aa$
        Dim i, i3, ns2 As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindBody = dat(1)
        If KindBody = 0 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KindSect = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KindConnect = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Nsect = dat(1)
            a4$ = "右捩れ角（°）"
        Else
            KindSect = 0
            Do
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                If dat(0) = "放射メッシュ数" Then
                    Nsect = dat(1)
                    Exit Do
                End If
            Loop
            a4$ = "放射メッシュ開始角（°）"
        End If
        ns2 = 2 * Nsect
        ReDim Xsect(ns2), Ysect(ns2), Zsect(ns2)

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        If dat(0) = a4$ Then
            AngleMesh = dat(1)
            If KindBody = 0 Then
                If KindSect = 1 Then
                    aa$ = sr.ReadLine
                    dat = aa.Split(",")
                    RatioSim = dat(1)
                End If
            Else
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                AngleMesh2 = dat(1)
            End If
        Else
            AngleMesh = 0
            AngleMesh2 = 360
            If KindBody = 0 And KindSect = 1 Then
                RatioSim = dat(1)
            End If
        End If

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Faum$ = dat(1)
        If KindSect = 2 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Fadd$ = dat(1)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KeLeft = dat(1)
            KpLeft = dat(2)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            KeRight = dat(1)
            KpRight = dat(2)
        Else
            KeLeft = 1 : KpLeft = 1
            KeRight = 1 : KpRight = 1
        End If

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        XBL = dat(1)
        YBL = dat(2)
        If KindSect > 1 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            XBR = dat(1)
            YBR = dat(2)
        End If

        If KindBody = 0 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            CosLX(1) = dat(1)
            CosLX(2) = dat(2)
            CosLX(3) = dat(3)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            CosLY(1) = dat(1)
            CosLY(2) = dat(2)
            CosLY(3) = dat(3)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            CosRX(1) = dat(1)
            CosRX(2) = dat(2)
            CosRX(3) = dat(3)
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            CosRY(1) = dat(1)
            CosRY(2) = dat(2)
            CosRY(3) = dat(3)

            If KindConnect > 0 Then
                i3 = 1
            Else
                i3 = ns2
            End If
            aa$ = sr.ReadLine
            For i = 0 To ns2 Step i3
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Xsect(i) = dat(1)
                Ysect(i) = dat(2)
                Zsect(i) = dat(3)
            Next i
        End If

        sr.Close()

    End Sub

    Sub AuzFileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim a1$, a2$, a3$, a4$, a5$, aa$
        Dim i, i3, ns2 As Integer

        a1$ = ""
        a2$ = ""
        Dim sw As New IO.StreamWriter(fxx$)
        sw.WriteLine(Title$)
        aa$ = "構造体種別（0:柱状体、1:軸回転体）" + "," + Str(KindBody)
        sw.WriteLine(aa$)
        If KindBody = 0 Then
            a1$ = "両端断面種類（0:同一、1:相似、2:異形）"
            a2$ = "両端面の接続方法（0:直線、1:曲線）"
            a3$ = "両端面間のメッシュ数"
            a4$ = "右捩れ角（°）"
            a5$ = "相似倍率"
        Else
            KindSect = 0
            a3$ = "放射メッシュ数"
            a4$ = "放射メッシュ開始角（°）"
            a5$ = "放射メッシュ終了角（°）"
        End If
        If KindBody = 0 Then
            aa$ = a1$ + "," + Str(KindSect)
            sw.WriteLine(aa$)
            aa$ = a2$ + "," + Str(KindConnect)
            sw.WriteLine(aa$)
        End If
        aa$ = a3$ + "," + Str(Nsect)
        sw.WriteLine(aa$)
        aa$ = a4$ + "," + Str(AngleMesh)
        sw.WriteLine(aa$)

        If KindBody = 0 Then
            If KindSect = 1 Then
                aa$ = a5$ + "," + Str(RatioSim)
                sw.WriteLine(aa$)
            End If
        Else
            aa$ = a5$ + "," + Str(AngleMesh2)
            sw.WriteLine(aa$)
        End If

        If KindBody = 0 Then
            a1$ = "[左端面のデータファイル名]"
            If KindSect > 1 Then
                a2$ = "[右端面のデータファイル名]"
            End If
            a3$ = "左端面内の基準位置ＸB、ＹB"
            If KindSect > 1 Then
                a4$ = "右端面内の基準位置ＸB、ＹB"
            End If
        Else
            a1$ = "[半断面のデータファイル名]"
            a3$ = "半断面内の基準位置ＸB、ＹB"
        End If

        aa$ = a1$ + ", " + Faum$
        sw.WriteLine(aa$)
        If KindSect > 1 Then
            aa$ = a2$ + "," + Fadd$
            sw.WriteLine(aa$)
            aa$ = "左端の基準要素番号、指定節点番号" + ","
            aa$ = aa$ + Str(KeLeft) + "," + Str(KpLeft)
            sw.WriteLine(aa$)
            aa$ = "右端の相当要素番号、相当指定節点番号" + ","
            aa$ = aa$ + Str(KeRight) + "," + Str(KpRight)
            sw.WriteLine(aa$)
        End If

        aa$ = a3$ + "," + Str(XBL) + "," + Str(YBL)
        sw.WriteLine(aa$)
        If KindSect > 1 Then
            aa$ = a4$ + "," + Str(XBR) + "," + Str(YBR)
            sw.WriteLine(aa$)
        End If

        If KindBody = 0 Then
            aa$ = "左端断面Ｘ軸の方向余弦" + "," + Str(CosLX(1))
            aa$ = aa$ + "," + Str(CosLX(2)) + "," + Str(CosLX(3))
            sw.WriteLine(aa$)
            aa$ = "左端断面Ｙ軸の方向余弦" + "," + Str(CosLY(1))
            aa$ = aa$ + "," + Str(CosLY(2)) + "," + Str(CosLY(3))
            sw.WriteLine(aa$)
            aa$ = "右端断面Ｘ軸の方向余弦" + "," + Str(CosRX(1))
            aa$ = aa$ + "," + Str(CosRX(2)) + "," + Str(CosRX(3))
            sw.WriteLine(aa$)
            aa$ = "右端断面Ｙ軸の方向余弦" + "," + Str(CosRY(1))
            aa$ = aa$ + "," + Str(CosRY(2)) + "," + Str(CosRY(3))
            sw.WriteLine(aa$)

            ns2 = 2 * Nsect
            If KindConnect > 0 Then
                i3 = 1
            Else
                i3 = ns2
            End If
            sw.WriteLine("各断面の位置データ（面番号、Ｘ、Ｙ、Ｚ）")
            For i = 0 To ns2 Step i3
                aa$ = Str$(i) + ",  " + Str$(Xsect(i))
                aa$ = aa$ + ",  " + Str$(Ysect(i))
                aa$ = aa$ + ",  " + Str$(Zsect(i))
                sw.WriteLine(aa$)
            Next i
        End If

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub AxisCheck(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmax As Single)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmax･･･1軸の最大長さ
        Dim ie, ii, ik, j, k, nmax As Integer
        Dim b1 As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        AxisMin(1) = Xdata(k)
                        AxisMin(2) = Ydata(k)
                        AxisMin(3) = Zdata(k)
                        For ii = 1 To 3
                            AxisMax(ii) = AxisMin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = Xdata(k)
                                Case 2
                                    b1 = Ydata(k)
                                Case 3
                                    b1 = Zdata(k)
                            End Select
                            If b1 < AxisMin(ii) Then
                                AxisMin(ii) = b1
                            ElseIf b1 > AxisMax(ii) Then
                                AxisMax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmax = AxisMax(1) - AxisMin(1)
        For ii = 2 To 3
            b1 = AxisMax(ii) - AxisMin(ii)
            If b1 > rlenmax Then
                rlenmax = b1
            End If
        Next ii

    End Sub

    Sub AxisCheck2(ByVal k1 As Integer, ByVal k2 As Integer, _
        ByRef xmin As Single, ByRef xmax As Single, _
        ByRef ymin As Single, ByRef ymax As Single)
        'ｽｸﾘｰﾝにおけるX、Y軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim ie, ik, k, j, nmax As Integer

        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = Xscreen(k) : xmax = xmin
                        ymin = Yscreen(k) : ymax = ymin
                    Else
                        If Xscreen(k) < xmin Then
                            xmin = Xscreen(k)
                        ElseIf Xscreen(k) > xmax Then
                            xmax = Xscreen(k)
                        End If
                        If Yscreen(k) < ymin Then
                            ymin = Yscreen(k)
                        ElseIf Yscreen(k) > ymax Then
                            ymax = Yscreen(k)
                        End If
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub AxisCheck2D(ByRef icheck As Integer, ByVal k1 As Integer,
        ByVal k2 As Integer, ByVal ratio As Single, ByRef xmin As Single,
        ByRef xmax As Single, ByRef ymin As Single, ByRef ymax As Single)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        'k1,k2･･･要素の開始、終了番号
        'ratio･･･作画直角回転なしの縦横比上限（指定値）
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim i, ie, ik, j As Integer
        Dim b1, b2, ratiob As Single
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To 8
                    i = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        xmin = X(i) : xmax = xmin
                        ymin = Y(i) : ymax = ymin
                    Else
                        If X(i) < xmin Then
                            xmin = X(i)
                        ElseIf X(i) > xmax Then
                            xmax = X(i)
                        End If
                        If Y(i) < ymin Then
                            ymin = Y(i)
                        ElseIf Y(i) > ymax Then
                            ymax = Y(i)
                        End If
                    End If
                Next j
            End If
        Next ie
        b1 = xmax - xmin
        b2 = ymax - ymin
        If b1 = 0 Then
            icheck = 0
        Else
            ratiob = b2 / b1
            If ratiob > ratio Then
                icheck = 1
            Else
                icheck = 0
            End If
        End If

    End Sub

    Sub AxisCheckM(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByRef rlenmax As Single, ByRef axmax() As Single, ByRef axmin() As Single)
        'ﾓﾃﾞﾙ座標X、Y、Z軸の最小・最大値のﾁｪｯｸ
        'k1,k2･･･要素の開始、終了番号
        'rlenmax･･･1軸の最大長さ
        'axmax()･･･X,Y,Z座標の最大値
        'axmin()･･･X,Y,Z座標の最小値
        Dim ie, ii, ik, j, k, nmax As Integer
        Dim b1 As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        ik = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    If ik = 0 Then
                        ik = 1
                        axmin(1) = Xmodel(k)
                        axmin(2) = Ymodel(k)
                        axmin(3) = Zmodel(k)
                        For ii = 1 To 3
                            axmax(ii) = axmin(ii)
                        Next ii
                    Else
                        For ii = 1 To 3
                            Select Case ii
                                Case 1
                                    b1 = Xmodel(k)
                                Case 2
                                    b1 = Ymodel(k)
                                Case 3
                                    b1 = Zmodel(k)
                            End Select
                            If b1 < axmin(ii) Then
                                axmin(ii) = b1
                            ElseIf b1 > axmax(ii) Then
                                axmax(ii) = b1
                            End If
                        Next ii
                    End If
                Next j
            End If
        Next ie

        rlenmax = axmax(1) - axmin(1)
        For ii = 2 To 3
            b1 = axmax(ii) - axmin(ii)
            If b1 > rlenmax Then
                rlenmax = b1
            End If
        Next ii

    End Sub

    Sub AxisValueTreat(ByRef xbuf() As Single, ByRef ybuf() As Single, ByRef zbuf() As Single)
        '座標値の端数処理
        'xbuf(),ybuf(),zbuf()･･･節点の座標
        Dim i, ib, ie, kind, nb As Integer
        Dim b2, rlenb As Single
        ReDim KdrawElem(Nelem)

        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie

        Call AxisCheck(1, Nelem, rlenb)
        'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        b2 = rlenb * 0.00002
        nb = 0
        i = Int(b2)
        If i > 0 Then
            kind = 0
            Do
                ib = CInt(b2)
                If ib < 10 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 0.1
                End If
            Loop
        Else
            kind = 1
            Do
                ib = CInt(b2)
                If ib > 0 Then
                    Exit Do
                Else
                    nb = nb + 1
                    b2 = b2 * 10
                End If
            Loop
        End If

        For i = 1 To Npoint
            xbuf(i) = Xdata(i)
            ybuf(i) = Ydata(i)
            zbuf(i) = Zdata(i)
        Next i
        For i = 1 To Npoint
            If Xdata(i) <> 0 Then
                Call RoundTreat(kind, nb, xbuf(i)) '丸め処理
            End If
            If Ydata(i) <> 0 Then
                Call RoundTreat(kind, nb, ybuf(i)) '丸め処理
            End If
            If Zdata(i) <> 0 Then
                Call RoundTreat(kind, nb, zbuf(i)) '丸め処理
            End If
        Next i

    End Sub

    Sub BandWidth2(ByRef mb As Integer, ByVal kcbuf() As Integer)
        'ﾊﾞﾝﾄﾞ幅の計算
        'mb･･･ﾊｰﾌﾊﾞﾝﾄﾞ幅
        Dim i, ie, ii, j, jj, k, mx, nmax As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        mx = 0
        For ie = 1 To Nelem
            For ii = 1 To nmax - 1
                i = NNelem(ie, ii)
                For jj = ii + 1 To nmax
                    j = NNelem(ie, jj)
                    k = Math.Abs(kcbuf(i) - kcbuf(j))
                    If k > mx Then mx = k
                Next jj
            Next ii
        Next ie
        mb = mx + 1

    End Sub

    Sub CenterCalc(ByRef xcb As Single, ByRef ycb As Single, ByRef zcb As Single)
        '構造中心の座標計算
        'xcb,ycb,zcb･･･構造中心の座標
        Dim i As Integer

        xcb = 0 : ycb = 0 : zcb = 0
        For i = 1 To Npoint
            xcb = xcb + Xdata(i)
            ycb = ycb + Ydata(i)
            zcb = zcb + Zdata(i)
        Next i
        xcb = xcb / Npoint
        ycb = ycb / Npoint
        zcb = zcb / Npoint

    End Sub

    Sub CoordiChange(ByVal rlenb As Single)
        '節点座標の視点座標への変換
        'rlenb･･･ﾓﾃﾞﾙ座標原点と視点座標との距離
        Dim ip As Integer
        For ip = 1 To Npoint
            Xview(ip) = Xmodel(ip)
            Yview(ip) = Zmodel(ip)
            Zview(ip) = Ymodel(ip) + rlenb
        Next ip

    End Sub

    Private Sub CrosPoint(ByRef alphab As Single, ByRef beta As Single, _
    ByVal eps As Single, ByVal xbuf() As Single, ByVal ybuf() As Single, _
    ByVal xbufc() As Single, ByVal ybufc() As Single)
        '2線分の交点計算
        'alphab,beta･･･各線分の交点位置比
        'xbuf(),ybuf()･･･基準側の2節点座標
        'xbufc(),ybufc()･･･比較側の2節点座標
        Dim a1, a2, b1, b2, b3, b4, bb, cc, dd, dist, distb As Single
        Dim eps2, f1, f2, xb, yb As Single

        alphab = -1
        eps2 = eps * eps
        a1 = ybuf(2) - ybuf(1)
        b1 = -(xbuf(2) - xbuf(1))
        dist = Math.Sqrt(a1 * a1 + b1 * b1)
        a2 = ybufc(2) - ybufc(1)
        b2 = -(xbufc(2) - xbufc(1))
        distb = Math.Sqrt(a2 * a2 + b2 * b2)
        If dist < eps Or distb < eps Then
            Exit Sub
        End If
        If Math.Abs(a1) < eps And Math.Abs(b2) < eps Then
            xb = xbufc(1)
            yb = ybuf(1)
            b1 = xb - xbuf(1)
            b2 = yb - ybufc(1)
            cc = b1 * (xb - xbuf(2))
            dd = b2 * (yb - ybufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        ElseIf Math.Abs(b1) < eps And Math.Abs(a2) < eps Then
            xb = xbuf(1)
            yb = ybufc(1)
            b1 = yb - ybuf(1)
            b2 = xb - xbufc(1)
            cc = b1 * (yb - ybuf(2))
            dd = b2 * (xb - xbufc(2))
            If cc < eps2 And dd < eps2 Then
                alphab = Math.Abs(b1) / dist
                beta = Math.Abs(b2) / distb
            End If
        Else
            f1 = a1 * xbuf(1) + b1 * ybuf(1)
            f2 = a2 * xbufc(1) + b2 * ybufc(1)
            bb = a1 * b2 - b1 * a2
            If Math.Abs(bb) > eps2 Then
                xb = (f1 * b2 - f2 * b1) / bb
                yb = (f2 * a1 - f1 * a2) / bb
                b1 = xb - xbuf(1)
                b2 = yb - ybuf(1)
                cc = b1 * (xb - xbuf(2))
                dd = b2 * (yb - ybuf(2))
                If cc < eps2 And dd < eps2 Then
                    b3 = xb - xbufc(1)
                    b4 = yb - ybufc(1)
                    cc = b3 * (xb - xbufc(2))
                    dd = b4 * (yb - ybufc(2))
                    If cc < eps2 And dd < eps2 Then
                        alphab = Math.Sqrt(b1 * b1 + b2 * b2) / dist
                        beta = Math.Sqrt(b3 * b3 + b4 * b4) / distb
                    End If
                End If
            End If
        End If

    End Sub

    Sub Debug1(ByVal nx As Integer)
        '作画面のﾃﾞﾊﾞｯｸﾞ
        '作画面ﾃﾞｰﾀを出力して停止
        'nx･･･作画面の数
        Dim aa$, fxx$
        Dim i, ii As Integer
        ii = FreeFile()
        fxx$ = DirOpen$ + "\SolidDebug1.dat"
        FileOpen(ii, fxx$, OpenMode.Output)
        aa$ = "作画No.、要素No.、面No."
        WriteLine(ii, aa$)
        For i = 1 To nx
            aa$ = Str$(i)
            aa$ = aa$ + ",  " + Str$(KEorder(i))
            aa$ = aa$ + ",  " + Str$(KSorder(i))
            PrintLine(ii, aa$)
        Next i
        FileClose(ii)
        Stop

    End Sub

    Sub DataCheckS(ByRef kh As Integer)
        '入力ﾃﾞｰﾀのｴﾗｰﾁｪｯｸ
        'kh･･･ｴﾗｰ有無（0：なし、-1：有り）
        Dim i, ie, ig, ih, ii, ip, j, jg As Integer
        Dim k, kg, nb, numg, nmax, nx As Integer
        Dim b1, b2, b3, dj, s, t, u As Single

        If KindElem = 0 Then
            nmax = 20
        End If

        kh = 0
        If Npoint < 20 Then
            kh = -1
            Exit Sub
        End If
        If Nelem < 1 Then
            kh = -1
            Exit Sub
        End If

        '要素のﾁｪｯｸ
        For ie = 1 To Nelem
            nb = NumMat(ie)
            If nb < 1 Then
                kh = -1
                Exit Sub
            End If
            For j = 1 To nmax
                nb = NNelem(ie, j)
                If nb < 1 Or nb > Npoint Then
                    kh = -1
                    Exit Sub
                End If
                If j < nmax Then
                    For k = j + 1 To nmax
                        nx = NNelem(ie, k)
                        If nb = nx Then
                            kh = -1
                            Exit Sub
                        End If
                    Next k
                End If
            Next j
        Next ie

        '節点のﾁｪｯｸ
        For ip = 1 To Npoint - 1
            For j = ip + 1 To Npoint
                b1 = Xdata(ip) - Xdata(j)
                b2 = Ydata(ip) - Ydata(j)
                b3 = Zdata(ip) - Zdata(j)
                If b1 = 0 And b2 = 0 And b3 = 0 Then
                    kh = -1
                    Exit Sub
                End If
            Next j
        Next ip

        '要素積分のﾁｪｯｸ
        For ie = 1 To Nelem
            For ii = 1 To nmax
                i = NNelem(ie, ii)
                XE(ii) = Xdata(i)
                YE(ii) = Ydata(i)
                ZE(ii) = Zdata(i)
            Next ii
            If KindElem = 0 Then
                Ngaus = 3
                PosGaus(3) = 0.77459667 : PosGaus(1) = -PosGaus(3) : PosGaus(2) = 0
                WeightG(1) = 0.55555556 : WeightG(3) = WeightG(1)
                WeightG(2) = 0.88888889
                numg = 0
                For ig = 1 To Ngaus
                    s = PosGaus(ig)
                    For jg = 1 To Ngaus
                        t = PosGaus(jg)
                        For kg = 1 To Ngaus
                            numg = numg + 1
                            u = PosGaus(kg)
                            Call ShapeFunc(s, t, u)         '形状関数の計算
                            Call JacobElem(ih, dj)    '要素のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
                            If ih < 0 Then
                                kh = -1
                                Exit Sub
                            End If
                        Next kg
                    Next jg
                Next ig
            End If
        Next ie

    End Sub

    Function DeleteMultiSpace(str As String) As String
        DeleteMultiSpace = str
        While InStr(DeleteMultiSpace, "  ") > 0
            DeleteMultiSpace = Replace(DeleteMultiSpace, "  ", " ")
        End While

    End Function

    Sub DfileInputU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾌｧｲﾙ名
        Dim aa$
        Dim i, j, nmax As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        aa$ = sr.ReadLine
        dat = aa.Split(",")
        KindElem = dat(1)
        If KindElem = 0 Then
            nmax = 20
        Else
            aa$ = "六面体要素以外は使用できません！"
            MsgBox(aa$, 0, "データエラー")
            sr.Close()
            Exit Sub
        End If
        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)
        If KindElem = 0 Then
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NEcross = dat(1)
            If NEcross > 0 Then
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                KindBase = dat(1)
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                Vbase = dat(1)
            End If
        End If

        ReDim NNelem(Nelem, 20), NumMat(Nelem)
        ReDim Xdata(Npoint), Ydata(Npoint), Zdata(Npoint)

        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(i) = dat(1)
            For j = 1 To 10
                NNelem(i, j) = dat(j + 1)
            Next j
            If nmax > 10 Then
                aa$ = sr.ReadLine
                dat = aa.Split(",")
                For j = 11 To nmax
                    NNelem(i, j) = dat(j - 11)
                Next j
            End If
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            Xdata(i) = dat(1)
            Ydata(i) = dat(2)
            Zdata(i) = dat(3)
        Next i

        sr.Close()

    End Sub

    Sub DfileOutU(ByVal fxx$)
        '入力ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･入力ﾃﾞｰﾀﾌｧｲﾙ名
        Dim a1$, aa$
        Dim i, j As Integer

        Dim sw As New IO.StreamWriter(fxx$)
        aa$ = "要素種類（0:六面体、1:四面体）" + "," + Str(KindElem)
        sw.WriteLine(aa$)
        sw.WriteLine(Title$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)
        If KindElem = 0 Then
            aa$ = "交差面の要素数" + "," + Str(NEcross)
            sw.WriteLine(aa$)
            If NEcross > 0 Then
                aa$ = "ベース構造種類（0:円柱、1:角柱）" + "," + Str(KindBase)
                sw.WriteLine(aa$)
                If KindBase = 0 Then
                    a1$ = "円柱の半径"
                Else
                    a1$ = "角柱側面のＹ座標"
                End If
                aa$ = a1$ + "," + Str(Vbase)
                sw.WriteLine(aa$)
            End If
            sw.WriteLine("要素データ (要素番号、材料番号、20節点番号)")
        End If

        For i = 1 To Nelem
            aa$ = Str$(i) + "," + Str$(NumMat(i))
            For j = 1 To 10
                aa$ = aa$ + "," + Str$(NNelem(i, j))
            Next j
            sw.WriteLine(aa$)
            If KindElem = 0 Then
                aa$ = "    " + Str$(NNelem(i, 11))
                For j = 12 To 20
                    aa$ = aa$ + "," + Str$(NNelem(i, j))
                Next j
                sw.WriteLine(aa$)
            End If
        Next i

        aa$ = "節点データ（節点番号、Ｘ座標、Ｙ座標、Ｚ座標）"
        sw.WriteLine(aa$)
        For i = 1 To Npoint
            aa$ = Str$(i)
            aa$ = aa$ + ",  " + Str$(Xdata(i))
            aa$ = aa$ + ",  " + Str$(Ydata(i))
            aa$ = aa$ + ",  " + Str$(Zdata(i))
            sw.WriteLine(aa$)
        Next i

        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub DfilePrepA()
        '構造ﾃﾞｰﾀﾌｧｲﾙ保存の前処理
        NEcross = 0
        Ngaus = 3

    End Sub

    Sub Dist2Set(ByVal kbuf() As Integer, ByRef dist2b As Single)
        '距離2乗の設定
        'kbuf()･･･面の節点番号
        'dist2b･･･距離2乗
        Dim i, ib, nmax As Integer
        Dim c1, xb, yb, zb As Single
        If KindElem = 0 Then
            nmax = 8
            c1 = 0.25
        End If
        xb = 0 : yb = 0 : zb = 0
        For i = 1 To nmax - 1 Step 2
            ib = kbuf(i)
            xb = xb + Xmodel(ib)
            yb = yb + Ymodel(ib)
            zb = zb + Zmodel(ib)
        Next i
        xb = c1 * xb
        yb = c1 * yb + Rlenmodel
        zb = c1 * zb - Eyeh
        dist2b = xb * xb + yb * yb + zb * zb

    End Sub

    Private Sub DistComp(ByRef ic As Integer, ByVal kbuf() As Integer)
        '距離比較
        'ic･･･基準面の距離区分（0:近い、1:遠い）
        'kbuf()･･･比較面の8節点番号
        Dim i, i1, i2, ib, icc, j, j1, j2, jb, nmax As Integer
        Dim alphab, b1, b2, b3, beta, c1, dist, distc, eps, xb, yb, zb As Single
        Dim xbuf(2), xbufc(2), ybuf(2), ybufc(2) As Single

        If KindElem = 0 Then
            nmax = 8
            c1 = 0.25
        End If
        ic = -1  '仮設定
        For i = 1 To nmax - 1 Step 2
            i1 = KsurfPoint(i)
            ib = i + 2
            If ib > nmax Then
                ib = 1
            End If
            i2 = KsurfPoint(ib)
            xbuf(1) = Xscreen(i1)
            ybuf(1) = Yscreen(i1)
            xbuf(2) = Xscreen(i2)
            ybuf(2) = Yscreen(i2)
            b1 = Xmodel(i1) - Xmodel(i2)
            b2 = Ymodel(i1) - Ymodel(i2)
            b3 = Zmodel(i1) - Zmodel(i2)
            eps = 0.0001 * Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
            For j = 1 To nmax - 1 Step 2
                j1 = kbuf(j)
                jb = j + 2
                If jb > nmax Then
                    jb = 1
                End If
                j2 = kbuf(jb)
                icc = 0
                If i1 = j1 Or i1 = j2 Then
                    icc = 1
                ElseIf i2 = j1 Or i2 = j2 Then
                    icc = 1
                End If
                If icc = 0 Then
                    xbufc(1) = Xscreen(j1)
                    ybufc(1) = Yscreen(j1)
                    xbufc(2) = Xscreen(j2)
                    ybufc(2) = Yscreen(j2)
                    Call CrosPoint(alphab, beta, eps, xbuf, ybuf, xbufc, ybufc)
                    '2線分の交点計算

                    If alphab >= 0 Then
                        b1 = 1 - alphab
                        xb = b1 * Xmodel(i1) + alphab * Xmodel(i2)
                        yb = b1 * Ymodel(i1) + alphab * Ymodel(i2) + Rlenmodel
                        zb = b1 * Zmodel(i1) + alphab * Zmodel(i2) - Eyeh
                        dist = xb * xb + yb * yb + zb * zb
                        b1 = 1 - beta
                        xb = b1 * Xmodel(j1) + beta * Xmodel(j2)
                        yb = b1 * Ymodel(j1) + beta * Ymodel(j2) + Rlenmodel
                        zb = b1 * Zmodel(j1) + beta * Zmodel(j2) - Eyeh
                        distc = xb * xb + yb * yb + zb * zb
                        If dist < distc Then
                            ic = 0
                        Else
                            ic = 1
                        End If
                        icc = -1
                        Exit For
                    End If
                End If
            Next j
            If icc < 0 Then
                Exit For
            End If
        Next i

        If ic < 0 Then
            xb = 0 : yb = 0 : zb = 0
            For i = 1 To nmax - 1 Step 2
                ib = KsurfPoint(i)
                xb = xb + Xmodel(ib)
                yb = yb + Ymodel(ib)
                zb = zb + Zmodel(ib)
            Next
            xb = c1 * xb
            yb = c1 * yb + Rlenmodel
            zb = c1 * zb - Eyeh
            dist = xb * xb + yb * yb + zb * zb
            xb = 0 : yb = 0 : zb = 0
            For i = 1 To nmax - 1 Step 2
                ib = kbuf(i)
                xb = xb + Xmodel(ib)
                yb = yb + Ymodel(ib)
                zb = zb + Zmodel(ib)
            Next
            xb = c1 * xb
            yb = c1 * yb + Rlenmodel
            zb = c1 * zb - Eyeh
            distc = xb * xb + yb * yb + zb * zb

            If dist < distc Then
                ic = 0
            Else
                ic = 1
            End If
        End If

    End Sub

    Sub DrawFocusRect(ByRef gr As Graphics)
        'ﾌｫｰｶｽ矩形の描画
        Dim rect As Rectangle
        Dim p1, p2 As Point

        If Xmouse(1) <= Xmouse(2) Then
            p1.X = Xmouse(1)
            p2.X = Xmouse(2)
        Else
            p1.X = Xmouse(2)
            p2.X = Xmouse(1)
        End If
        If Ymouse(1) <= Ymouse(2) Then
            p1.Y = Ymouse(1)
            p2.Y = Ymouse(2)
        Else
            p1.Y = Ymouse(2)
            p2.Y = Ymouse(1)
        End If
        rect.X = p1.X
        rect.Y = p1.Y
        rect.Width = p2.X - p1.X
        rect.Height = p2.Y - p1.Y

        gr.DrawRectangle(Pens.DarkGray, rect)   '矩形の描画

    End Sub

    Sub DrawScale(ByRef icheck As Integer, ByVal dpixb As Single, _
        ByVal dpiyb As Single, ByVal xlen As Single, ByVal ylen As Single, _
        ByVal xmin As Single, ByVal xmax As Single, _
        ByVal ymin As Single, ByVal ymax As Single)
        '作画倍率の計算
        'icheck･･･作画直角回転要否（0:回転なし、1:直角回転要）
        'dpixb･･･X軸方向のdpi値
        'dpiyb･･･Y軸方向のdpi値
        'xlen･･･作画領域のX方向長さ
        'ylen･･･作画領域のY方向長さ
        'xmin･･･X座標最小値
        'xmax･･･X座標最大値
        'ymin･･･Y座標最小値
        'ymax･･･Y座標最大値
        Dim b1, b2, cx, cy As Single

        Cfix = dpiyb / dpixb

        b1 = xmax - xmin : b2 = ymax - ymin
        Select Case icheck
            Case 0
                If Cfix < 1 Then
                    If b1 <> 0 Then
                        cx = xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2 / Cfix
                    End If
                Else
                    If b1 <> 0 Then
                        cx = Cfix * xlen / b1
                    End If
                    If b2 <> 0 Then
                        cy = ylen / b2
                    End If
                End If
                If b1 = 0 Then
                    XYscale = cy
                ElseIf b2 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
            Case 1
                If Cfix < 1 Then
                    If b2 <> 0 Then
                        cx = xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1 / Cfix
                    End If
                Else
                    If b2 <> 0 Then
                        cx = Cfix * xlen / b2
                    End If
                    If b1 <> 0 Then
                        cy = ylen / b1
                    End If
                End If
                If b2 = 0 Then
                    XYscale = cy
                ElseIf b1 = 0 Then
                    XYscale = cx
                ElseIf cy < cx Then
                    XYscale = cy
                Else
                    XYscale = cx
                End If
        End Select

    End Sub

    Sub Elem2DPrep()
        '構造図作画準備
        Dim ie As Integer
        Dim pa, qa, ratio As Single
        ReDim KdrawElem(Nelem)

        KE1 = 1
        KE2 = Nelem
        KnumSize = 8
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
        Next ie

        GoTo Skip10
        ratio = 1.2
        Kaxis = 0
        Call AxisCheck2D(Kaxis, 1, Nelem, ratio, XaxisMin, pa, YaxisMin, qa)
        'X、Y軸の最小・最大値と作画直角回転要否のﾁｪｯｸ
Skip10:

    End Sub

    Sub Elem3DPrep()
        '構造図作画準備
        Dim ie As Integer
        ReDim KdrawElem(Nelem)
        ReDim Xmodel(Npoint), Ymodel(Npoint), Zmodel(Npoint)
        ReDim Xscreen(Npoint), Yscreen(Npoint)
        ReDim Xview(Npoint), Yview(Npoint), Zview(Npoint)

        KE1 = 1
        KE2 = Nelem
        KnumDisp = 0
        KnumSize = 8
        Nmat = 1
        For ie = KE1 To KE2
            KdrawElem(ie) = 1
            If NumMat(ie) > Nmat Then
                Nmat = NumMat(ie)
            End If
        Next ie
        KE3DCut(1) = 0

        KrearTreat = 1
        KdispENum = 0
        KdispPNum = 0
        KdispSNum = 0
        KdispMat = 0

        Rlenmax = 0
        DistFactor = 0      '作画ﾊﾟﾗﾒｰﾀｰのｸﾘｱ

    End Sub

    Sub ElemRelation(ByRef icc As Integer, ByVal ie As Integer, ByVal ib As Integer)
        '要素関係の判別
        'icc･･･関係の有無（0:なし、1:有り）
        'ie･･･元の要素番号
        'ib･･･対象の要素番号
        Dim i, j, k As Integer
        If KindElem = 0 Then
            For i = 1 To 7 Step 2
                k = NNelem(ie, i)
                For j = 1 To 7 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
                For j = 13 To 19 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
            Next i
            For i = 13 To 19 Step 2
                k = NNelem(ie, i)
                For j = 1 To 7 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
                For j = 13 To 19 Step 2
                    If k = NNelem(ib, j) Then
                        icc = 1
                        Exit Sub
                    End If
                Next j
            Next i
        End If

    End Sub

    Sub ElemSearch(ByRef k1 As Integer, ByRef k2 As Integer)
        '拡大作画する要素の探索
        '全節点が指定領域にある要素を採用、作画要素なしの場合は元のﾃﾞｰﾀを保存
        'k1,k2･･･要素の開始、終了番号
        Dim ic, ie, j, k, kmin, kmax, nmax As Integer
        Dim b1, b2 As Single
        Dim kbuf(Nelem) As Integer
        If KindElem = 0 Then
            nmax = 20
        End If
        ic = 0
        For ie = 1 To Nelem
            kbuf(ie) = KdrawElem(ie)
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    b1 = Xscreen(k) - Xmouse(1)
                    b2 = Xscreen(k) - Xmouse(2)
                    If b1 >= 0 And b2 <= 0 Then
                        b1 = Yscreen(k) - Ymouse(2)
                        b2 = Yscreen(k) - Ymouse(1)
                        If b1 >= 0 And b2 <= 0 Then
                            If j = nmax Then
                                KdrawElem(ie) = 1
                                If ic = 0 Then
                                    ic = 1
                                    kmin = ie
                                    kmax = ie
                                ElseIf ie < kmin Then
                                    kmin = ie
                                ElseIf ie > kmax Then
                                    kmax = ie
                                End If
                            End If
                        Else
                            KdrawElem(ie) = 0
                            Exit For
                        End If
                    Else
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie
        k1 = kmin
        k2 = kmax
        If k1 = 0 Then
            For ie = 1 To Nelem
                KdrawElem(ie) = kbuf(ie)
            Next ie
        End If

    End Sub

    Sub EyeCheck(ByVal k1 As Integer, ByVal k2 As Integer)
        '視線ﾁｪｯｸ（面の中心位置で判定）
        'k1,k2･･･開始・終了要素番号
        Dim ie, j, k, kb, kmax, nb, nmax As Integer
        Dim b1, b2, b3, b4, xb, yb, zb As Single
        Dim eyevec(3), planevec(3) As Single
        Dim xbufb(3), ybufb(3), zbuf(3) As Single

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
            nb = 4
        End If
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If

                        xb = 0 : yb = 0 : zb = 0
                        For j = 2 To nmax Step 2
                            kb = KsurfPoint(j)
                            xb = xb + Xmodel(kb)
                            yb = yb + Ymodel(kb)
                            zb = zb + Zmodel(kb) - Eyeh
                            Select Case j
                                Case 2
                                    xbufb(2) = Xmodel(kb)
                                    ybufb(2) = Ymodel(kb)
                                    zbuf(2) = Zmodel(kb) - Eyeh
                                Case 4
                                    xbufb(3) = Xmodel(kb)
                                    ybufb(3) = Ymodel(kb)
                                    zbuf(3) = Zmodel(kb) - Eyeh
                            End Select
                        Next j
                        xbufb(1) = xb / nb
                        ybufb(1) = yb / nb
                        zbuf(1) = zb / nb

                        '逆視線ﾍﾞｸﾄﾙの計算
                        b1 = -xbufb(1)
                        b2 = -(Rlenmodel + ybufb(1))
                        b3 = -zbuf(1)
                        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
                        eyevec(1) = b1 / b4
                        eyevec(2) = b2 / b4
                        eyevec(3) = b3 / b4

                        '面の法線ﾍﾞｸﾄﾙの計算
                        b1 = (ybufb(2) - ybufb(1)) * (zbuf(3) - zbuf(1))
                        b1 = b1 - (zbuf(2) - zbuf(1)) * (ybufb(3) - ybufb(1))
                        b2 = (zbuf(2) - zbuf(1)) * (xbufb(3) - xbufb(1))
                        b2 = b2 - (xbufb(2) - xbufb(1)) * (zbuf(3) - zbuf(1))
                        b3 = (xbufb(2) - xbufb(1)) * (ybufb(3) - ybufb(1))
                        b3 = b3 - (ybufb(2) - ybufb(1)) * (xbufb(3) - xbufb(1))
                        b4 = Math.Sqrt(b1 * b1 + b2 * b2 + b3 * b3)
                        planevec(1) = b1 / b4
                        planevec(2) = b2 / b4
                        planevec(3) = b3 / b4

                        '逆視線ﾍﾞｸﾄﾙと面法線ﾍﾞｸﾄﾙのｽｶﾗｰ積計算
                        b1 = 0
                        For j = 1 To 3
                            b1 = b1 + eyevec(j) * planevec(j)
                        Next j
                        If b1 <= 0 Then
                            KSelem(ie, k) = -1   '視線外
                        End If
                    End If
                Next k
            End If
        Next ie

    End Sub

    Sub FileName(ByRef fxx$, ByRef fxb$)
        'ﾌｧｲﾙ名fxx$の拡張子なしの部分fxb$を取り出す
        Dim i As Integer
        i = InStr(fxx$, ".")
        If i > 0 Then
            fxb$ = Left$(fxx$, i - 1)
        Else
            fxb$ = fxx$
        End If
    End Sub

    Sub InnerCheck2(ByVal k1 As Integer, ByVal k2 As Integer)
        '内面ﾁｪｯｸ
        'k1,k2･･･開始・終了要素番号
        Dim ib, ic, icc, ie, ik, ik1, j, jj, jx, k, kk, kmax, ksb, ksmax, m, nmax As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            ksmax = 4
            nmax = 7
            ReDim KinnerLine(12 * Nelem) '内部辺のﾌﾗｯｸﾞ
        End If
        Dim ksbuf(Nelem, kmax) As Integer    'ﾁｪｯｸ済みﾌﾗｯｸﾞ
        For ie = k1 To k2 - 1
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If k = 1 Then
                        icc = 0
                    End If
                    If ksbuf(ie, k) = 0 Then
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, k, KsurfPoint)  '面の設定
                        End If
                        ik = 0
                        For ib = ie + 1 To k2
                            If KdrawElem(ib) > 0 Then
                                If icc = 0 Then
                                    Call ElemRelation(icc, ie, ib)    '要素関係の判別
                                End If
                                If icc > 0 Then
                                    For kk = 1 To kmax
                                        If ksbuf(ib, kk) = 0 Then
                                            If KindElem = 0 Then
                                                Call SurfaceSet(ib, kk, kbuf)   '面の設定
                                            End If

                                            ik1 = KsurfPoint(1)
                                            ic = 0
                                            For j = 1 To nmax Step 2
                                                If ik1 = kbuf(j) Then
                                                    jx = j
                                                    For jj = 3 To nmax Step 2
                                                        jx = jx - 2
                                                        If jx < 1 Then
                                                            jx = nmax
                                                        End If
                                                        If KsurfPoint(jj) <> kbuf(jx) Then
                                                            ic = 1
                                                            Exit For
                                                        End If
                                                    Next jj
                                                    If ic = 0 Then
                                                        ik = 1
                                                        KSelem(ie, k) = 1
                                                        KSelem(ib, kk) = 1
                                                        ksbuf(ie, k) = 1
                                                        ksbuf(ib, kk) = 1
                                                        For m = 1 To ksmax
                                                            ksb = KSideLine(ie, k, m)
                                                            KinnerLine(ksb) = 1
                                                            ksb = KSideLine(ib, kk, m)
                                                            KinnerLine(ksb) = 1
                                                        Next m
                                                    End If
                                                    Exit For
                                                End If
                                            Next j
                                            If ik = 1 Then
                                                Exit For
                                            End If
                                        End If
                                    Next kk
                                    If ik = 1 Then
                                        Exit For
                                    End If
                                End If
                            Else
                                icc = 0
                            End If
                        Next ib
                    End If
                Next k
            End If
        Next ie

    End Sub

    Sub JacobElem(ByRef ih As Integer, ByRef dj As Single)
        '要素のﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの計算
        'ih･･･ｴﾗｰ有無（0:なし、-1:有り）
        'dj･･･ﾔｺﾋﾞｱﾝ ﾏﾄﾘｯｸｽの行列式値
        Dim i, ii, j, nmax As Integer
        Dim b1 As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        For i = 1 To 3
            For j = 1 To 3
                RinvJ(i, j) = 0
                RJmat(i, j) = 0
            Next j
        Next i
        For ii = 1 To nmax
            For j = 1 To 3
                Select Case j
                    Case 1
                        b1 = XE(ii)
                    Case 2
                        b1 = YE(ii)
                    Case 3
                        b1 = ZE(ii)
                End Select
                RJmat(1, j) = RJmat(1, j) + Ds(ii) * b1
                RJmat(2, j) = RJmat(2, j) + Dt(ii) * b1
                RJmat(3, j) = RJmat(3, j) + Du(ii) * b1
            Next j
        Next ii
        RinvJ(1, 1) = RJmat(2, 2) * RJmat(3, 3) - RJmat(2, 3) * RJmat(3, 2)
        RinvJ(1, 2) = RJmat(1, 3) * RJmat(3, 2) - RJmat(1, 2) * RJmat(3, 3)
        RinvJ(1, 3) = RJmat(1, 2) * RJmat(2, 3) - RJmat(1, 3) * RJmat(2, 2)
        RinvJ(2, 1) = RJmat(2, 3) * RJmat(3, 1) - RJmat(2, 1) * RJmat(3, 3)
        RinvJ(2, 2) = RJmat(1, 1) * RJmat(3, 3) - RJmat(1, 3) * RJmat(3, 1)
        RinvJ(2, 3) = RJmat(1, 3) * RJmat(2, 1) - RJmat(1, 1) * RJmat(2, 3)
        RinvJ(3, 1) = RJmat(2, 1) * RJmat(3, 2) - RJmat(2, 2) * RJmat(3, 1)
        RinvJ(3, 2) = RJmat(1, 2) * RJmat(3, 1) - RJmat(1, 1) * RJmat(3, 2)
        RinvJ(3, 3) = RJmat(1, 1) * RJmat(2, 2) - RJmat(1, 2) * RJmat(2, 1)
        b1 = RJmat(1, 1) * RinvJ(1, 1) + RJmat(1, 2) * RinvJ(2, 1)
        dj = b1 + RJmat(1, 3) * RinvJ(3, 1)
        If dj > 0 Then
            ih = 0
        Else
            ih = -1
        End If

    End Sub

    Function KSideLine(ByVal ib As Integer, ByVal kb As Integer, _
        ByVal kk As Integer) As Integer
        '辺番号の照合
        'ib･･･要素番号
        'kb･･･面番号
        'kk･･･面の辺番号
        Dim ksb As Integer

        If KindElem = 0 Then    '六面体
            Select Case kb
                Case 1
                    ksb = kk
                Case 2
                    ksb = 13 - kk
                Case 3
                    Select Case kk
                        Case 1
                            ksb = 5
                        Case 2
                            ksb = 9
                        Case 3
                            ksb = 6
                        Case 4
                            ksb = 1
                    End Select
                Case 4
                    Select Case kk
                        Case 1
                            ksb = 7
                        Case 2
                            ksb = 11
                        Case 3
                            ksb = 8
                        Case 4
                            ksb = 3
                    End Select
                Case 5
                    Select Case kk
                        Case 1
                            ksb = 6
                        Case 2
                            ksb = 10
                        Case 3
                            ksb = 7
                        Case 4
                            ksb = 2
                    End Select
                Case 6
                    Select Case kk
                        Case 1
                            ksb = 8
                        Case 2
                            ksb = 12
                        Case 3
                            ksb = 5
                        Case 4
                            ksb = 4
                    End Select
            End Select
            KSideLine = 12 * (ib - 1) + ksb
        End If

    End Function

    Sub MeshCorrect3()
        'ﾒｯｼｭ訂正3
        Dim i, j, jmax, k, nb As Integer

        'jmax = 27
        jmax = 20
        Dim jbuf(jmax), kbuf(Npoint) As Integer
        For i = 1 To Nelem
            For j = 1 To jmax
                k = NNelem(i, j)
                kbuf(k) = 1
            Next j
        Next i
        nb = 0
        For i = 1 To Npoint
            If kbuf(i) > 0 Then
                nb = nb + 1
                kbuf(i) = nb
                Xdata(nb) = Xdata(i)
                Ydata(nb) = Ydata(i)
                Zdata(nb) = Zdata(i)
            End If
        Next i
        Npoint = nb
        For i = 1 To Nelem
            For j = 1 To jmax
                k = NNelem(i, j)
                NNelem(i, j) = kbuf(k)
            Next j
        Next i

        For i = 1 To Nelem
            For j = 1 To 20
                jbuf(j) = NNelem(i, j)
            Next j
            For j = 1 To 20
                Select Case j
                    Case 1
                        k = 1
                    Case 2
                        k = 10
                    Case 3
                        k = 4
                    Case 4
                        k = 14
                    Case 5
                        k = 3
                    Case 6
                        k = 12
                    Case 7
                        k = 2
                    Case 8
                        k = 9
                    Case 9
                        k = 11
                    Case 10
                        k = 16
                    Case 11
                        k = 15
                    Case 12
                        k = 13
                    Case 13
                        k = 5
                    Case 14
                        k = 18
                    Case 15
                        k = 8
                    Case 16
                        k = 20
                    Case 17
                        k = 7
                    Case 18
                        k = 19
                    Case 19
                        k = 6
                    Case 20
                        k = 17
                End Select
                NNelem(i, j) = jbuf(k)
            Next j
        Next i

        ReDim NumMat(Nelem)
        For i = 1 To Nelem
            NumMat(i) = 1
        Next i

    End Sub

    Sub MiniSort2(ByVal kbuf() As Integer, ByRef nsort() As Integer, ByVal work() As Single, _
        ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        'ﾐﾆｿｰﾄ2
        '同じ変数値の場合に、第2座標値でソート
        'kbuf()･･･ｿｰﾄ順の座標軸番号
        'nsort()･･･元の変数番号のｿｰﾄ順番号
        'work()･･･ｿｰﾄ変数値
        'xbuf(),ybuf()･･･節点の座標
        Dim i, j, jb, jx, k, k1, k2, kx, nb, n1, n2, nx As Integer
        Dim b1 As Single
        Dim nsbuf(10), nsort2(10), nsort3(10) As Integer
        Dim work2(10), work3(10) As Single

        i = 1
        Do
            If work(i) = work(i + 1) Then
                b1 = work(i)
                n1 = i
                For j = i + 1 To Npoint
                    If b1 = work(j) Then
                        n2 = j
                    Else
                        Exit For
                    End If
                Next j
                nx = n2 - n1 + 1
                ReDim nsort2(nx), work2(nx)
                jb = 0
                For j = n1 To n2
                    jb = jb + 1
                    nsort2(jb) = jb
                    nb = nsort(j)
                    Select Case kbuf(2)
                        Case 1
                            work2(jb) = xbuf(nb)
                        Case 2
                            work2(jb) = ybuf(nb)
                        Case 3
                            work2(jb) = zbuf(nb)
                    End Select
                Next j
                Call ShellSortStd(nx, nsort2, work2)    'ｼｪｯﾙｿｰﾄ基本

                k = 1
                Do
                    If work2(k) = work2(k + 1) Then
                        b1 = work2(k)
                        k1 = k
                        For j = k + 1 To nx
                            If b1 = work2(j) Then
                                k2 = j
                            Else
                                Exit For
                            End If
                        Next j
                        kx = k2 - k1 + 1
                        ReDim nsort3(kx), work3(kx), nsbuf(kx)
                        jb = 0
                        For j = k1 To k2
                            jb = jb + 1
                            nsbuf(jb) = nsort2(j)
                            nsort3(jb) = jb
                            nb = nsort(n1 + nsort2(j) - 1)
                            Select Case kbuf(3)
                                Case 1
                                    work3(jb) = xbuf(nb)
                                Case 2
                                    work3(jb) = ybuf(nb)
                                Case 3
                                    work3(jb) = zbuf(nb)
                            End Select
                        Next j
                        Call ShellSortStd(kx, nsort3, work3)    'ｼｪｯﾙｿｰﾄ基本
                        For j = 1 To kx
                            jb = k1 + j - 1
                            nsort2(jb) = nsbuf(nsort3(j))
                        Next j
                        k = k2 + 1
                    Else
                        k = k + 1
                    End If
                Loop While k < nx

                For j = 1 To nx
                    jb = n1 + nsort2(j) - 1
                    nsort2(j) = nsort(jb)
                Next j
                For j = 1 To nx
                    jx = n1 + j - 1
                    nsort(jx) = nsort2(j)
                Next j
                i = n2 + 1
            Else
                i = i + 1
            End If
        Loop While i < Npoint

    End Sub

    Sub ModelTrans(ByVal nb As Integer, ByRef xbuf() As Single, _
    ByRef ybuf() As Single, ByRef zbuf() As Single)
        'ﾓﾃﾞﾙ節点座標のｱﾌｨﾝ変換
        'nb･･･節点数
        'xbuf(),ybuf(),zbuf()･･･節点のX,Y,Z座標
        Dim ip, j, k As Integer
        Dim b1 As Single
        Dim buf(3) As Single
        For ip = 1 To nb
            buf(1) = xbuf(ip)
            buf(2) = ybuf(ip)
            buf(3) = zbuf(ip)
            For k = 1 To 3
                b1 = 0
                For j = 1 To 3
                    b1 = b1 + TransMat(k, j) * buf(j)
                Next j
                b1 = b1 + TransMat(k, 4)
                Select Case k
                    Case 1
                        xbuf(ip) = b1
                    Case 2
                        ybuf(ip) = b1
                    Case 3
                        zbuf(ip) = b1
                End Select
            Next k
        Next ip

    End Sub

    Sub MouseCalc2()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                If Kaxis = 0 Then
                    xb = (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale / Cfix + XaxisMin
                    yb = (xa - Xlower) / XYscale + YaxisMin
                End If
            Else
                If Kaxis = 0 Then
                    xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                    yb = (Yupper - ya) / XYscale + YaxisMin
                Else
                    xb = (ya - Ylower) / XYscale + XaxisMin
                    yb = Cfix * (xa - Xlower) / XYscale + YaxisMin
                End If
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub MouseCalc()
        'ﾏｳｽ点のﾓﾃﾞﾙ座標への変換
        Dim j As Integer
        Dim xa, xb, ya, yb As Single
        For j = 1 To 2
            xa = Xmouse(j)
            ya = Ymouse(j)
            If Cfix < 1 Then
                xb = (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale / Cfix + YaxisMin
            Else
                xb = Cfix * (xa - Xlower) / XYscale + XaxisMin
                yb = (Yupper - ya) / XYscale + YaxisMin
            End If
            Xmouse(j) = xb
            Ymouse(j) = yb
        Next j

    End Sub

    Sub NumCheck(ByRef ic As Integer, ByRef i1 As Integer, _
        ByRef i2 As Integer, ByRef j1 As Integer, _
        ByRef j2 As Integer)
        '要素辺の重なりのﾁｪｯｸ
        'ic･･･要素辺の重なり有無（0:なし、1:有り）
        'i1･･･第1の辺の一端の節点番号
        'i2･･･第1の辺の他端の節点番号
        'j1･･･第2の辺の一端の節点番号
        'j2･･･第2の辺の他端の節点番号
        Dim j1b, j2b As Integer
        Dim b1, b2, b3 As Single
        ic = 0
        b1 = i1 - i2
        b2 = j1 - j2
        b3 = b1 * b2
        If b3 < 0 Then
            j1b = j2 : j2b = j1
        Else
            j1b = j1 : j2b = j2
        End If
        If (i1 = j1b) And (i2 = j2b) Then
            ic = 1
        End If

    End Sub

    Sub OptFileIn()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\SolDMenu.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            KdataBackup = Val(sr.ReadLine)
            Kscreen = Val(sr.ReadLine)
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub OptFileOut()
        'ｵﾌﾟｼｮﾝ･ﾌｧｲﾙ書き出し
        Dim ftmp$

        ftmp$ = DirOpen$ + "\SolDMenu.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(KdataBackup)
        sw.WriteLine(Kscreen)
        sw.Close()

    End Sub

    Sub OuterReset(ByVal k1 As Integer, ByVal k2 As Integer)
        '外郭面の再設定
        'k1,k2･･･開始・終了要素番号
        Dim ie, k, kmax As Integer

        If KindElem = 0 Then
            kmax = 6
        End If
        For ie = k1 To k2
            For k = 1 To kmax
                If KSelem(ie, k) < 0 Then
                    KSelem(ie, k) = 0
                End If
            Next k
        Next ie
    End Sub

    Sub PlaneCutElem()
        '面切断の要素選択
        Dim ic, ie, j, k, nmax As Integer
        Dim b1, std As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        If KE3DCut(2) > 0 Then
            std = VE3DCut(1) - VE3DCut(2)
        Else
            std = VE3DCut(1) + VE3DCut(2)
        End If

        For ie = 1 To Nelem
            If KdrawElem(ie) > 0 Then
                ic = 0
                For j = 1 To nmax
                    k = NNelem(ie, j)
                    Select Case KE3DCut(1)
                        Case 1
                            b1 = Xdata(k)
                        Case 2
                            b1 = Ydata(k)
                        Case 3
                            b1 = Zdata(k)
                    End Select
                    If KE3DCut(2) > 0 Then
                        If b1 < std Then
                            ic = 1
                        End If
                    Else
                        If b1 > std Then
                            ic = 1
                        End If
                    End If
                    If ic > 0 Then
                        KdrawElem(ie) = 0
                        Exit For
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub PaintExe(ByRef gr As Graphics, ByVal ncount As Integer, _
    ByVal colorx As Color, ByVal kind As Integer)
        '塗りつぶし実行
        'gr･･･ｸﾞﾗﾌｨｯｸｽ･ｵﾌﾞｼﾞｪｸﾄ名
        'ncount･･･頂点数
        'colorx･･･境界および塗りつぶしの色
        'kind･･･境界作画の有無（0:なし、1:有り）
        Dim i As Integer
        Dim xydraw(ncount - 1) As Point
        Dim myBrush As New SolidBrush(colorx)

        For i = 1 To ncount
            xydraw(i - 1).X = Xdraw(i)
            xydraw(i - 1).Y = Ydraw(i)
        Next i
        gr.FillPolygon(myBrush, xydraw)

        If kind > 0 Then
            gr.DrawPolygon(Pens.Black, xydraw)
        End If

    End Sub

    Sub PerspecCalc(ByVal k1 As Integer, ByVal k2 As Integer, _
    ByVal rlenscreen As Single)
        'ｽｸﾘｰﾝへの透視座標の計算
        'k1,k2･･･要素の開始、終了番号
        'rlenscreen･･･透視座標原点からｽｸﾘｰﾝまでの距離
        Dim ib, ie, j, nmax As Integer
        Dim b1 As Single
        Dim kbuf(Npoint) As Integer

        If KindElem = 0 Then
            nmax = 20
        End If
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For j = 1 To nmax
                    ib = NNelem(ie, j)
                    If kbuf(ib) = 0 Then
                        b1 = rlenscreen / Zview(ib)
                        Xscreen(ib) = b1 * Xview(ib)
                        Yscreen(ib) = b1 * (Yview(ib) - Eyeh)
                        kbuf(ib) = 1
                    End If
                Next j
            End If
        Next ie

    End Sub

    Sub PnumImproveNew(ByRef mb As Integer, ByRef mbx As Integer, _
        ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        '節点順序の改良
        'kh･･･改良の有無（0:なし、1:有り）
        'mb･･･改良後のﾊﾞﾝﾄﾞ幅
        'mbx･･･改良前のﾊﾞﾝﾄﾞ幅
        Dim i, ib, ic, ie, j, k, k1, k2, mbb, nmax As Integer
        Dim rlenb As Single
        Dim kbuf(3), kbufb(3), kcbuf(Npoint) As Integer
        Dim rlbuf(3) As Single

        If KindElem = 0 Then
            nmax = 20
        End If
        For ie = 1 To Nelem
            KdrawElem(ie) = 1
        Next ie
        Call AxisCheck(1, Nelem, rlenb)     'X、Y、Z軸の最小・最大値のﾁｪｯｸ

        mbb = 0
        For k = 1 To 3
            kbuf(1) = k
            Select Case k
                Case 1
                    k1 = 2
                    k2 = 3
                Case 2
                    k1 = 3
                    k2 = 1
                Case 3
                    k1 = 1
                    k2 = 2
            End Select
            rlbuf(k1) = AxisMax(k1) - AxisMin(k1)
            rlbuf(k2) = AxisMax(k2) - AxisMin(k2)
            If rlbuf(k1) > rlbuf(k2) Then
                kbuf(2) = k1
                kbuf(3) = k2
            Else
                kbuf(2) = k2
                kbuf(3) = k1
            End If

            Call ShellSort2(kbuf, kcbuf, xbuf, ybuf, zbuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            ic = 0
            If k = 1 Or mb < mbb Then
                ic = 1
            End If
            If ic > 0 Then
                For i = 1 To 3
                    kbufb(i) = kbuf(i)
                Next i
                mbb = mb
            End If
        Next k

        If mbb < mbx Then
            For i = 1 To 3
                kbuf(i) = kbufb(i)
            Next i
            Call ShellSort2(kbuf, kcbuf, xbuf, ybuf, zbuf)    'ｼｪﾙｿｰﾄ2
            Call BandWidth2(mb, kcbuf)      'ﾊﾞﾝﾄﾞ幅の計算
            For i = 1 To Npoint
                Kcomp(i) = kcbuf(i)
            Next i
            For i = 1 To Npoint
                xbuf(i) = Xdata(i)
                ybuf(i) = Ydata(i)
                zbuf(i) = Zdata(i)
            Next i
            For i = 1 To Npoint
                ib = Kcomp(i)
                Xdata(ib) = xbuf(i)
                Ydata(ib) = ybuf(i)
                Zdata(ib) = zbuf(i)
            Next i
            For ie = 1 To Nelem
                For j = 1 To nmax
                    i = NNelem(ie, j)
                    NNelem(ie, j) = Kcomp(i)
                Next j
            Next ie
        Else
            mb = mbx
        End If

    End Sub

    Sub ReNumberEP(ByVal kind As Integer, ByVal kround As Integer, _
    ByVal m1 As Integer, ByVal m2 As Integer)
        '基本断面の要素･節点番号の付番改良
        'kind･･･中心穴の有無（0:なし、1:有り）
        'kround･･･全周の有無（0:なし、1:有り）
        'm1･･･円周方向の分割数
        'm2･･･半径方向の分割数
        Dim i, ib, ie, iex, j, jj, k, nb As Integer
        Dim nbuf(Npoint), nnbuf(Nelem, 8) As Integer
        Dim xbuf(Npoint), ybuf(Npoint) As Single

        If kind = 0 Then
            nbuf(1) = 1
            xbuf(1) = X(1)
            ybuf(1) = Y(1)
            nb = 1
        Else
            nb = 0
        End If
        If kround = 1 Then '全周の場合
            ib = 0
            For i = m2 To 1 Step -1
                For j = 1 To m1
                    ie = i + m2 * (j - 1)
                    For jj = 1 To 8
                        k = NNelem(ie, jj)
                        If nbuf(k) = 0 Then
                            nb = nb + 1
                            nbuf(k) = nb
                            xbuf(nb) = X(k)
                            ybuf(nb) = Y(k)
                        End If
                    Next jj
                Next j
                For j = 1 To m1
                    ie = i + m2 * (j - 1)
                    ib = ib + 1
                    For jj = 1 To 8
                        nnbuf(ib, jj) = NNelem(ie, jj)
                    Next jj
                Next j
            Next i
            For ie = 1 To Nelem
                For jj = 1 To 8
                    k = nnbuf(ie, jj)
                    NNelem(ie, jj) = nbuf(k)
                Next jj
            Next ie
            For i = 1 To Npoint
                X(i) = xbuf(i)
                Y(i) = ybuf(i)
            Next i
        Else
            If kind = 0 Then
                '最内部の節点番号変更
                For i = 1 To m1
                    ib = m2 * (i - 1)
                    iex = ib
                    j = m2
                    ie = ib + j
                    For jj = 1 To 8
                        k = NNelem(ie, jj)
                        If nbuf(k) = 0 Then
                            nb = nb + 1
                            nbuf(k) = nb
                            xbuf(nb) = X(k)
                            ybuf(nb) = Y(k)
                        End If
                    Next jj
                Next i
            End If

            For i = 1 To m1
                ib = m2 * (i - 1)
                iex = ib
                For j = m2 To 1 Step -1
                    ie = ib + j
                    iex = iex + 1
                    For jj = 1 To 8
                        k = NNelem(ie, jj)
                        If nbuf(k) = 0 Then
                            nb = nb + 1
                            nbuf(k) = nb
                            xbuf(nb) = X(k)
                            ybuf(nb) = Y(k)
                        End If
                        nnbuf(iex, jj) = nbuf(k)
                    Next jj
                Next j
                iex = ib
                For j = m2 To 1 Step -1
                    ie = ib + j
                    iex = iex + 1
                    For jj = 1 To 8
                        NNelem(iex, jj) = nnbuf(iex, jj)
                    Next jj
                Next j
            Next i
            For i = 1 To Npoint
                X(i) = xbuf(i)
                Y(i) = ybuf(i)
            Next i
        End If

    End Sub

    Sub RotateCalc(ByVal kind As Integer, ByRef theta As Single)
        '回転角の計算
        'kind･･･象限番号
        'theta･･･回転角（左ねじ正、°）
        Dim ix, iy, kb As Integer
        Dim b1 As Single

        ix = Xmouse(2) - Xmouse(1)
        iy = Ymouse(2) - Ymouse(1)
        Select Case kind
            Case 1
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                End If
            Case 2
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                Else
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                End If
            Case 3
                If ix > 0 Then
                    If iy >= 0 Then
                        kb = 1
                    Else
                        kind = 0
                    End If
                Else
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = -1
                    End If
                End If
            Case 4
                If ix > 0 Then
                    If iy >= 0 Then
                        kind = 0
                    Else
                        kb = 1
                    End If
                Else
                    If iy >= 0 Then
                        kb = -1
                    Else
                        kind = 0
                    End If
                End If
        End Select
        b1 = Math.Sqrt(ix * ix + iy * iy)
        theta = kb * 3.0 * b1 / 50.0

    End Sub

    Sub RoundTreat(ByVal kind As Integer, ByVal nb As Integer, ByRef zb As Single)
        '丸め処理
        'kind･･･丸め区分（0:1以上、1:1未満）
        'nb･･･丸め位置の桁数
        'zb･･･対象変数
        Dim ib As Integer
        Dim b1 As Single

        If kind = 0 Then
            b1 = zb * 10 ^ (-nb)
            ib = CInt(b1)
            zb = ib * 10 ^ nb
        Else
            b1 = zb * 10 ^ nb
            ib = CInt(b1)
            zb = ib * 10 ^ (-nb)
        End If

    End Sub

    Sub SfileInputU(ByRef fxx$)
        '断面生成ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)

        ReDim NNelem(Nelem, 8), NumMat(Nelem), X(Npoint), Y(Npoint)

        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(i) = dat(1)
            For j = 1 To 8
                NNelem(i, j) = dat(j + 1)
            Next j
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            X(i) = dat(1)
            Y(i) = dat(2)
        Next i

        sr.Close()

    End Sub

    Sub SfileOut(ByRef fxx$)
        '断面生成ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'fxx$･･･ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, ii, j As Integer
        ii = FreeFile()
        FileOpen(ii, fxx$, OpenMode.Output)
        WriteLine(ii, Title$)
        WriteLine(ii, "節点数", Npoint)
        WriteLine(ii, "要素数", Nelem)

        WriteLine(ii, "要素データ (要素番号、材料番号、8節点番号)")
        For i = 1 To Nelem
            aa$ = Str$(i) + ",   " + Str$(NumMat(i))
            For j = 1 To 8
                aa$ = aa$ + ",   " + Str$(NNelem(i, j))
            Next j
            PrintLine(ii, aa$)
        Next i

        WriteLine(ii, "節点データ（節点番号、Ｘ座標、Ｙ座標）")
        For i = 1 To Npoint
            aa$ = Str$(i) + ",   " + Str$(X(i)) + ",   " + Str$(Y(i))
            PrintLine(ii, aa$)
        Next i
        FileClose(ii)

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub SfileOutU(ByRef fxx$)
        '断面生成ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, j As Integer

        Dim sw As New IO.StreamWriter(fxx$)

        sw.WriteLine(Title$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)

        sw.WriteLine("要素データ (要素番号、材料番号、8節点番号)")
        For i = 1 To Nelem
            aa$ = Str$(i) + ",   " + Str$(NumMat(i))
            For j = 1 To 8
                aa$ = aa$ + ",   " + Str$(NNelem(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        sw.WriteLine("節点データ（節点番号、Ｘ座標、Ｙ座標）")
        For i = 1 To Npoint
            aa$ = Str$(i) + ",   " + Str$(X(i)) + ",   " + Str$(Y(i))
            sw.WriteLine(aa$)
        Next i
        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub ShapeFunc(ByVal s As Single, ByVal t As Single, _
        ByVal u As Single)
        '形状関数の計算
        's･･･要素のξ座標
        't･･･要素のη座標
        'u･･･要素のζ座標
        Dim s1, s1m, s2m, t1, t1m, t2m, u1, u1m, u2m As Single
        s1 = 1 + s
        s1m = 1 - s
        s2m = 1 - s * s
        t1 = 1 + t
        t1m = 1 - t
        t2m = 1 - t * t
        u1 = 1 + u
        u1m = 1 - u
        u2m = 1 - u * u
        Shape(1) = s1m * t1m * u1 * (-s - t + u - 2) / 8
        Shape(2) = s2m * t1m * u1 / 4
        Shape(3) = s1 * t1m * u1 * (s - t + u - 2) / 8
        Shape(4) = s1 * t2m * u1 / 4
        Shape(5) = s1 * t1 * u1 * (s + t + u - 2) / 8
        Shape(6) = s2m * t1 * u1 / 4
        Shape(7) = s1m * t1 * u1 * (-s + t + u - 2) / 8
        Shape(8) = s1m * t2m * u1 / 4
        Shape(9) = s1m * t1m * u2m / 4
        Shape(10) = s1 * t1m * u2m / 4
        Shape(11) = s1 * t1 * u2m / 4
        Shape(12) = s1m * t1 * u2m / 4
        Shape(13) = s1m * t1m * u1m * (-s - t - u - 2) / 8
        Shape(14) = s2m * t1m * u1m / 4
        Shape(15) = s1 * t1m * u1m * (s - t - u - 2) / 8
        Shape(16) = s1 * t2m * u1m / 4
        Shape(17) = s1 * t1 * u1m * (s + t - u - 2) / 8
        Shape(18) = s2m * t1 * u1m / 4
        Shape(19) = s1m * t1 * u1m * (-s + t - u - 2) / 8
        Shape(20) = s1m * t2m * u1m / 4
        Ds(1) = t1m * u1 * (s + t - u + 1 + s) / 8
        Ds(2) = -t1m * u1 * s / 2
        Ds(3) = t1m * u1 * (s - t + u - 1 + s) / 8
        Ds(4) = t2m * u1 / 4
        Ds(5) = t1 * u1 * (s + t + u - 1 + s) / 8
        Ds(6) = -t1 * u1 * s / 2
        Ds(7) = t1 * u1 * (s - t - u + 1 + s) / 8
        Ds(8) = -t2m * u1 / 4
        Ds(9) = -t1m * u2m / 4
        Ds(10) = t1m * u2m / 4
        Ds(11) = t1 * u2m / 4
        Ds(12) = -t1 * u2m / 4
        Ds(13) = t1m * u1m * (s + t + u + 1 + s) / 8
        Ds(14) = -t1m * u1m * s / 2
        Ds(15) = t1m * u1m * (s - t - u - 1 + s) / 8
        Ds(16) = t2m * u1m / 4
        Ds(17) = t1 * u1m * (s + t - u - 1 + s) / 8
        Ds(18) = -t1 * u1m * s / 2
        Ds(19) = t1 * u1m * (s - t + u + 1 + s) / 8
        Ds(20) = -t2m * u1m / 4
        Dt(1) = s1m * u1 * (s + t - u + 1 + t) / 8
        Dt(2) = -s2m * u1 / 4
        Dt(3) = s1 * u1 * (-s + t - u + 1 + t) / 8
        Dt(4) = -s1 * u1 * t / 2
        Dt(5) = s1 * u1 * (s + t + u - 1 + t) / 8
        Dt(6) = s2m * u1 / 4
        Dt(7) = s1m * u1 * (-s + t + u - 1 + t) / 8
        Dt(8) = -s1m * u1 * t / 2
        Dt(9) = -s1m * u2m / 4
        Dt(10) = -s1 * u2m / 4
        Dt(11) = s1 * u2m / 4
        Dt(12) = s1m * u2m / 4
        Dt(13) = s1m * u1m * (s + t + u + 1 + t) / 8
        Dt(14) = -s2m * u1m / 4
        Dt(15) = s1 * u1m * (-s + t + u + 1 + t) / 8
        Dt(16) = -s1 * u1m * t / 2
        Dt(17) = s1 * u1m * (s + t - u - 1 + t) / 8
        Dt(18) = s2m * u1m / 4
        Dt(19) = s1m * u1m * (-s + t - u - 1 + t) / 8
        Dt(20) = -s1m * u1m * t / 2
        Du(1) = s1m * t1m * (-s - t + u - 1 + u) / 8
        Du(2) = s2m * t1m / 4
        Du(3) = s1 * t1m * (s - t + u - 1 + u) / 8
        Du(4) = s1 * t2m / 4
        Du(5) = s1 * t1 * (s + t + u - 1 + u) / 8
        Du(6) = s2m * t1 / 4
        Du(7) = s1m * t1 * (-s + t + u - 1 + u) / 8
        Du(8) = s1m * t2m / 4
        Du(9) = -s1m * t1m * u / 2
        Du(10) = -s1 * t1m * u / 2
        Du(11) = -s1 * t1 * u / 2
        Du(12) = -s1m * t1 * u / 2
        Du(13) = s1m * t1m * (s + t + u + 1 + u) / 8
        Du(14) = -s2m * t1m / 4
        Du(15) = s1 * t1m * (-s + t + u + 1 + u) / 8
        Du(16) = -s1 * t2m / 4
        Du(17) = s1 * t1 * (-s - t + u + 1 + u) / 8
        Du(18) = -s2m * t1 / 4
        Du(19) = s1m * t1 * (s - t + u + 1 + u) / 8
        Du(20) = -s1m * t2m / 4
    End Sub

    Sub ShapeFunc2(ByVal s As Single, ByVal t As Single)
        '形状関数の計算
        's･･･要素のξ座標
        't･･･要素のη座標
        Dim b1, b2, b3, s2, ss, st, t2, tt As Single
        s2 = s * 2 : t2 = t * 2
        ss = s * s : tt = t * t : st = s * t
        b1 = ss * t : b2 = s * tt : b3 = st * 2
        Shape(1) = (-1 + st + ss + tt - b1 - b2) / 4
        Shape(2) = (1 - t - ss + b1) / 2
        Shape(3) = (-1 - st + ss + tt - b1 + b2) / 4
        Shape(4) = (1 + s - tt - b2) / 2
        Shape(5) = (-1 + st + ss + tt + b1 + b2) / 4
        Shape(6) = (1 + t - ss - b1) / 2
        Shape(7) = (-1 - st + ss + tt + b1 - b2) / 4
        Shape(8) = (1 - s - tt + b2) / 2
        Ds(1) = (t + s2 - b3 - tt) / 4
        Ds(2) = -s + st
        Ds(3) = (-t + s2 - b3 + tt) / 4
        Ds(4) = (1 - tt) / 2
        Ds(5) = (t + s2 + b3 + tt) / 4
        Ds(6) = -s - st
        Ds(7) = (-t + s2 + b3 - tt) / 4
        Ds(8) = (-1 + tt) / 2
        Dt(1) = (s + t2 - ss - b3) / 4
        Dt(2) = (-1 + ss) / 2
        Dt(3) = (-s + t2 - ss + b3) / 4
        Dt(4) = -t - st
        Dt(5) = (s + t2 + ss + b3) / 4
        Dt(6) = (1 - ss) / 2
        Dt(7) = (-s + t2 + ss - b3) / 4
        Dt(8) = -t + st
    End Sub

    Private Sub ShellSort2(ByVal kbuf() As Integer, ByRef kcbuf() As Integer, _
        ByVal xbuf() As Single, ByVal ybuf() As Single, ByVal zbuf() As Single)
        'ｼｪﾙｿｰﾄ
        Dim i As Integer
        Dim nsort(Npoint) As Integer
        Dim work(Npoint) As Single

        Select Case kbuf(1)
            Case 1
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Xdata(i)
                Next i
            Case 2
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Ydata(i)
                Next i
            Case 3
                For i = 1 To Npoint
                    nsort(i) = i
                    work(i) = Zdata(i)
                Next i
        End Select

        Call ShellSortStd(Npoint, nsort, work)    'ｼｪｯﾙｿｰﾄ基本

        Call MiniSort2(kbuf, nsort, work, xbuf, ybuf, zbuf)    'ﾐﾆｿｰﾄ

        For i = 1 To Npoint
            kcbuf(nsort(i)) = i
        Next i

    End Sub

    Sub ShellSortStd(ByVal nx As Integer, ByRef nsort() As Integer, ByRef work() As Single)
        'ｼｪﾙｿｰﾄ基本
        'nx･･･変数の数
        'nsort()･･ｿｰﾄ順の元変数の番号
        'work()･･･ｿｰﾄ順の変数値
        Dim i, ib, j, m As Integer
        Dim varb As Single

        m = nx
        Do While m > 1
            m = m \ 2
            For j = m + 1 To nx
                varb = work(j)
                ib = nsort(j)
                i = j - m
                Do While work(i) >= varb
                    work(i + m) = work(i)
                    nsort(i + m) = nsort(i)
                    i = i - m
                    If i <= 0 Then
                        Exit Do
                    End If
                Loop
                work(i + m) = varb
                nsort(i + m) = ib
            Next j
        Loop

    End Sub

    Sub SnFileInputU(ByRef fxx$)
        '空間断面の生成ﾃﾞｰﾀﾌｧｲﾙの読み込み
        'UTF-8･･･BOMなし
        'fxx$･･･ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, j As Integer

        Dim sr As New IO.StreamReader(fxx$)
        Dim dat() As String

        Title$ = sr.ReadLine
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Npoint = dat(1)
        aa$ = sr.ReadLine
        dat = aa.Split(",")
        Nelem = dat(1)

        ReDim NNelem(Nelem, 8), NumMat(Nelem), X(Npoint), Y(Npoint), Z(Npoint)

        aa$ = sr.ReadLine
        For i = 1 To Nelem
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            NumMat(i) = dat(1)
            For j = 1 To 8
                NNelem(i, j) = dat(j + 1)
            Next j
        Next i

        aa$ = sr.ReadLine
        For i = 1 To Npoint
            aa$ = sr.ReadLine
            dat = aa.Split(",")
            X(i) = dat(1)
            Y(i) = dat(2)
            Z(i) = dat(3)
        Next i

        sr.Close()

    End Sub

    Sub SnFileOutU(ByRef fxx$)
        '空間の断面生成ﾃﾞｰﾀﾌｧｲﾙへの書き出し
        'UTF-8･･･BOMなし
        'fxx$･･･ﾃﾞｰﾀﾌｧｲﾙ名
        Dim aa$
        Dim i, j As Integer

        Dim sw As New IO.StreamWriter(fxx$)
        sw.WriteLine(Title$)
        aa$ = "節点数" + "," + Str(Npoint)
        sw.WriteLine(aa$)
        aa$ = "要素数" + "," + Str(Nelem)
        sw.WriteLine(aa$)

        sw.WriteLine("要素データ (要素番号、材料番号、8節点番号)")
        For i = 1 To Nelem
            aa$ = Str$(i) + ",   " + Str$(NumMat(i))
            For j = 1 To 8
                aa$ = aa$ + ",   " + Str$(NNelem(i, j))
            Next j
            sw.WriteLine(aa$)
        Next i

        sw.WriteLine("節点データ（節点番号、Ｘ座標、Ｙ座標、Z座標）")
        For i = 1 To Npoint
            aa$ = Str$(i) + ",   " + Str$(X(i)) + ",   " + Str$(Y(i))
            aa$ = aa$ + ",   " + Str(Z(i))
            sw.WriteLine(aa$)
        Next i
        sw.Close()

        aa$ = "データをファイルへ保存しました。"
        MsgBox(aa$, 0, "ファイル保存完了")

    End Sub

    Sub SurfaceDrawC(ByRef gr As Graphics, ByVal k1 As Integer, _
        ByVal k2 As Integer, ByVal kind As Integer)
        '面の作画（材料別色分け）
        'k1,k2･･･要素の開始・終了番号
        'kind･･･面外周の作画有無（0:なし、1:有り）
        Dim aa$
        Dim ie, j, jmax, k, kb, kk, kmax, nmax, nb As Integer
        Dim xb, xd, xi, xj, yb, yd, yi, yj As Single
        Dim fnt As Font = New Font("ＭＳ ゴシック", KnumSize)
        Dim fnt2 As Font = New Font("ＭＳ ゴシック", KnumSb, FontStyle.Bold)

        If KindElem = 0 Then
            jmax = 6
            kmax = 8
            nmax = 20
        End If
        If KrearTreat > 0 Then  '隠面処理有りの場合
            For nb = 1 To NSdraw
                ie = KEorder(nb)
                kb = KSorder(nb)
                If KindElem = 0 Then
                    Call SurfaceSet(ie, kb, KsurfPoint) '面の設定
                End If
                For k = 1 To kmax
                    kk = KsurfPoint(k)
                    xd = Xscreen(kk)
                    yd = Yscreen(kk)
                    Call XYCalc(xd, yd)     'X,Yの作画座標計算
                    Xdraw(k) = xd
                    Ydraw(k) = yd
                Next k
                kk = (NumMat(ie) - 1) Mod 7
                Select Case kk
                    Case 0
                        Call PaintExe(gr, kmax, Color.White, kind)
                    Case 1
                        Call PaintExe(gr, kmax, Color.LightBlue, kind)
                    Case 2
                        Call PaintExe(gr, kmax, Color.LightGreen, kind)
                    Case 3
                        Call PaintExe(gr, kmax, Color.LightCyan, kind)
                    Case 4
                        Call PaintExe(gr, kmax, Color.LightPink, kind)
                    Case 5
                        Call PaintExe(gr, kmax, Color.LightYellow, kind)
                    Case 6
                        Call PaintExe(gr, kmax, Color.LightGray, kind)
                End Select
                If KnumDisp > 0 Then
                    xb = 0 : yb = 0
                    For k = 1 To kmax
                        xb = xb + Xdraw(k)
                        yb = yb + Ydraw(k)
                        If KdispPNum > 0 Then
                            kk = KsurfPoint(k)
                            xd = Xdraw(k)
                            yd = Ydraw(k)
                            aa$ = LTrim$(Str$(kk))
                            gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                        End If
                    Next k
                    If KdispSNum > 0 Then
                        xd = xb / kmax
                        yd = yb / kmax
                        aa$ = "(" + LTrim$(Str$(kb)) + ")"
                        gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                    End If
                    If KdispENum > 0 Then
                        xb = 0 : yb = 0
                        For j = 1 To nmax
                            kb = NNelem(ie, j)
                            xd = Xscreen(kb)
                            yd = Yscreen(kb)
                            Call XYCalc(xd, yd) 'X,Yの作画座標計算
                            xb = xb + xd
                            yb = yb + yd
                        Next j
                        xd = xb / nmax
                        yd = yb / nmax
                        aa$ = LTrim$(Str$(ie))
                        gr.DrawString(aa$, fnt2, Brushes.Black, xd, yd)
                    End If
                End If
            Next nb
        Else
            For ie = k1 To k2
                If KdrawElem(ie) > 0 Then
                    For j = 1 To jmax
                        If KindElem = 0 Then
                            Call SurfaceSet(ie, j, KsurfPoint)  '面の設定
                        End If
                        kk = KsurfPoint(kmax)
                        xi = Xscreen(kk)
                        yi = Yscreen(kk)
                        Call XYCalc(xi, yi) 'X,Yの作画座標計算
                        xb = 0 : yb = 0
                        For k = 1 To kmax
                            kk = KsurfPoint(k)
                            xj = Xscreen(kk)
                            yj = Yscreen(kk)
                            Call XYCalc(xj, yj) 'X,Yの作画座標計算
                            xb = xb + xj
                            yb = yb + yj
                            gr.DrawLine(Pens.Black, xi, yi, xj, yj)
                            xi = xj
                            yi = yj
                        Next k
                        If KnumDisp > 0 And KdispSNum > 0 Then
                            xj = xb / kmax
                            yj = yb / kmax
                            aa$ = "(" + LTrim$(Str$(j)) + ")"
                            gr.DrawString(aa$, fnt, Brushes.Black, xj, yj)
                        End If
                    Next j
                End If
            Next ie
            If KnumDisp > 0 Then
                For ie = k1 To k2
                    If KdrawElem(ie) > 0 Then
                        xb = 0 : yb = 0
                        For j = 1 To nmax
                            kb = NNelem(ie, j)
                            xi = Xscreen(kb)
                            yi = Yscreen(kb)
                            Call XYCalc(xi, yi) 'X,Yの作画座標計算
                            If KdispPNum > 0 Then
                                aa$ = LTrim$(Str$(kb))
                                gr.DrawString(aa$, fnt, Brushes.Black, xi, yi)
                            End If
                            xb = xb + xi
                            yb = yb + yi
                        Next j
                        If KdispENum > 0 Then
                            xj = xb / nmax
                            yj = yb / nmax
                            aa$ = LTrim$(Str$(ie))
                            gr.DrawString(aa$, fnt2, Brushes.Black, xj, yj)
                        End If
                    End If
                Next ie
            End If
        End If

        If KrearTreat > 0 And k1 <= KmouseElem And KmouseElem <= k2 Then
            j = 0
            For nb = 1 To NSdraw
                ie = KEorder(nb)
                kb = KSorder(nb)
                If ie = KmouseElem And kb = KmouseSurf Then
                    j = 1
                    Exit For
                End If
            Next nb
            If j > 0 Then
                If KindElem = 0 Then
                    Call SurfaceSet(ie, kb, KsurfPoint)  '面の設定
                End If
                For k = 1 To kmax
                    kk = KsurfPoint(k)
                    xd = Xscreen(kk)
                    yd = Yscreen(kk)
                    Call XYCalc(xd, yd)     'X,Yの作画座標計算
                    Xdraw(k) = xd
                    Ydraw(k) = yd
                Next k
                xb = 0 : yb = 0
                For k = 1 To kmax
                    xb = xb + Xdraw(k)
                    yb = yb + Ydraw(k)
                    kk = KsurfPoint(k)
                    xd = Xdraw(k)
                    yd = Ydraw(k)
                    aa$ = LTrim$(Str$(kk))
                    gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                Next k
                xd = xb / kmax
                yd = yb / kmax
                aa$ = "(" + LTrim$(Str$(kb)) + ")"
                gr.DrawString(aa$, fnt, Brushes.Black, xd, yd)
                xb = 0 : yb = 0
                For j = 1 To nmax
                    kb = NNelem(ie, j)
                    xd = Xscreen(kb)
                    yd = Yscreen(kb)
                    Call XYCalc(xd, yd) 'X,Yの作画座標計算
                    xb = xb + xd
                    yb = yb + yd
                Next j
                xd = xb / nmax
                yd = yb / nmax
                aa$ = LTrim$(Str$(ie))
                gr.DrawString(aa$, fnt2, Brushes.Black, xd, yd)
            End If
        End If

    End Sub

    Sub SurfaceSet(ByVal ib As Integer, ByVal kb As Integer, _
    ByRef kbuf() As Integer)
        '面の設定
        'ib･･･要素番号
        'kb･･･面番号
        'kbuf()･･･面の節点番号
        Select Case kb
            Case 1
                kbuf(1) = NNelem(ib, 1)
                kbuf(2) = NNelem(ib, 2)
                kbuf(3) = NNelem(ib, 3)
                kbuf(4) = NNelem(ib, 4)
                kbuf(5) = NNelem(ib, 5)
                kbuf(6) = NNelem(ib, 6)
                kbuf(7) = NNelem(ib, 7)
                kbuf(8) = NNelem(ib, 8)
            Case 2
                kbuf(1) = NNelem(ib, 13)
                kbuf(2) = NNelem(ib, 20)
                kbuf(3) = NNelem(ib, 19)
                kbuf(4) = NNelem(ib, 18)
                kbuf(5) = NNelem(ib, 17)
                kbuf(6) = NNelem(ib, 16)
                kbuf(7) = NNelem(ib, 15)
                kbuf(8) = NNelem(ib, 14)
            Case 3
                kbuf(1) = NNelem(ib, 1)
                kbuf(2) = NNelem(ib, 9)
                kbuf(3) = NNelem(ib, 13)
                kbuf(4) = NNelem(ib, 14)
                kbuf(5) = NNelem(ib, 15)
                kbuf(6) = NNelem(ib, 10)
                kbuf(7) = NNelem(ib, 3)
                kbuf(8) = NNelem(ib, 2)
            Case 4
                kbuf(1) = NNelem(ib, 5)
                kbuf(2) = NNelem(ib, 11)
                kbuf(3) = NNelem(ib, 17)
                kbuf(4) = NNelem(ib, 18)
                kbuf(5) = NNelem(ib, 19)
                kbuf(6) = NNelem(ib, 12)
                kbuf(7) = NNelem(ib, 7)
                kbuf(8) = NNelem(ib, 6)
            Case 5
                kbuf(1) = NNelem(ib, 3)
                kbuf(2) = NNelem(ib, 10)
                kbuf(3) = NNelem(ib, 15)
                kbuf(4) = NNelem(ib, 16)
                kbuf(5) = NNelem(ib, 17)
                kbuf(6) = NNelem(ib, 11)
                kbuf(7) = NNelem(ib, 5)
                kbuf(8) = NNelem(ib, 4)
            Case 6
                kbuf(1) = NNelem(ib, 7)
                kbuf(2) = NNelem(ib, 12)
                kbuf(3) = NNelem(ib, 19)
                kbuf(4) = NNelem(ib, 20)
                kbuf(5) = NNelem(ib, 13)
                kbuf(6) = NNelem(ib, 9)
                kbuf(7) = NNelem(ib, 1)
                kbuf(8) = NNelem(ib, 8)
        End Select

    End Sub

    Sub SurfSort(ByVal k1 As Integer, ByVal k2 As Integer)
        '作画面のｿｰﾃｨﾝｸﾞ
        'k1,k2･･･開始・終了要素番号
        Dim i1, i2, ic, ie, ii, ik1, ik2, j, k, kmax, nmax, nb As Integer
        Dim kbuf(8) As Integer

        If KindElem = 0 Then
            kmax = 6
            nmax = 8
        End If
        NSdraw = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        NSdraw = NSdraw + 1
                    End If
                Next k
            End If
        Next ie
        ReDim KEorder(NSdraw), KSorder(NSdraw)
        nb = 0
        For ie = k1 To k2
            If KdrawElem(ie) > 0 Then
                For k = 1 To kmax
                    If KSelem(ie, k) = 0 Then
                        nb = nb + 1
                        KEorder(nb) = ie
                        KSorder(nb) = k
                    End If
                Next k
            End If
        Next ie

        For ii = 1 To NSdraw - 1
            i1 = KEorder(ii)
            ik1 = KSorder(ii)
            If KindElem = 0 Then
                Call SurfaceSet(i1, ik1, KsurfPoint)    '面の設定
            End If

            For j = ii + 1 To NSdraw
                i2 = KEorder(j)
                ik2 = KSorder(j)
                If KindElem = 0 Then
                    Call SurfaceSet(i2, ik2, kbuf)      '面の設定
                End If

                Call DistComp(ic, kbuf)    '距離比較
                If ic = 0 Then
                    KEorder(ii) = i2
                    KSorder(ii) = ik2
                    KEorder(j) = i1
                    KSorder(j) = ik1
                    i1 = i2
                    ik1 = ik2
                    For k = 1 To nmax
                        KsurfPoint(k) = kbuf(k)
                    Next k
                End If
            Next j
        Next ii

    End Sub

    Sub TmpFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$

        ftmp$ = DirOpen$ + "\SolidData.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            Faum$ = sr.ReadLine
            Faun$ = sr.ReadLine
            Fauz$ = sr.ReadLine
            Fauc$ = sr.ReadLine
            Fm$ = sr.ReadLine
            Fmesh$ = sr.ReadLine
            Fy$ = sr.ReadLine
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub TmpFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        ftmp$ = DirOpen$ + "\SolidData.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        sw.WriteLine(Faum$)
        sw.WriteLine(Faun$)
        sw.WriteLine(Fauz$)
        sw.WriteLine(Fauc$)
        sw.WriteLine(Fm$)
        sw.WriteLine(Fmesh$)
        sw.WriteLine(Fy$)
        sw.Close()

    End Sub

    Sub TmatMulti(ByRef tr1buf(,) As Single, ByRef tr2buf(,) As Single)
        '変換行列の掛算
        'tr1buf(,)･･･最初の変換行列
        'tr2buf(,)･･･次の変換行列および結果の変化行列
        Dim i, j, k As Integer
        Dim b1 As Single
        Dim buf(4, 4) As Single

        For i = 1 To 4
            For j = 1 To 4
                buf(i, j) = tr2buf(i, j)
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf(i, k) * tr1buf(k, j)
                Next k
                tr2buf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalA(ByVal xs As Single, ByVal ys As Single, _
    ByVal zs As Single, ByVal xr As Single, ByVal yr As Single, _
    ByVal zr As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        '先にZ,Y,X軸回りの回転、最後に座標軸方向の平行移動
        'xs,ys,zs･･･X,Y,Z方向移動量
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i
        buf1(1, 4) = xs
        buf1(2, 4) = ys
        buf1(3, 4) = zs

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + TransMat(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                TransMat(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalD(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
        ByRef trbuf(,) As Single)
        'ﾓﾃﾞﾙの合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = -b2
        buf2(3, 2) = b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = b2
        buf3(2, 2) = 1
        buf3(3, 1) = -b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = -b2
        buf4(2, 1) = b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub TmatrixCalE(ByVal xr As Single, ByVal yr As Single, ByVal zr As Single, _
        ByRef trbuf(,) As Single)
        '座標軸の合成ｱﾌｨﾝ変換行列算出
        'Z,Y,X軸回りの回転
        'xr,yr,zr･･･X,Y,Z軸回りの回転角（右ねじ正、°）
        'trbuf(,)･･･変換ﾏﾄﾘｯｸｽ
        Dim i, j, k As Integer
        Dim b1, b2, thetax, thetay, thetaz As Single
        Dim buf1(4, 4), buf2(4, 4), buf3(4, 4), buf4(4, 4) As Single

        b1 = Pai / 180
        thetax = xr * b1
        thetay = yr * b1
        thetaz = zr * b1

        For i = 1 To 4
            buf1(i, i) = 1
        Next i

        b1 = Math.Cos(thetax)
        b2 = Math.Sin(thetax)
        buf2(1, 1) = 1
        buf2(2, 2) = b1
        buf2(2, 3) = b2
        buf2(3, 2) = -b2
        buf2(3, 3) = b1
        buf2(4, 4) = 1

        b1 = Math.Cos(thetay)
        b2 = Math.Sin(thetay)
        buf3(1, 1) = b1
        buf3(1, 3) = -b2
        buf3(2, 2) = 1
        buf3(3, 1) = b2
        buf3(3, 3) = b1
        buf3(4, 4) = 1

        b1 = Math.Cos(thetaz)
        b2 = Math.Sin(thetaz)
        buf4(1, 1) = b1
        buf4(1, 2) = b2
        buf4(2, 1) = -b2
        buf4(2, 2) = b1
        buf4(3, 3) = 1
        buf4(4, 4) = 1

        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf2(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + trbuf(i, k) * buf3(k, j)
                Next k
                buf1(i, j) = b1
            Next j
        Next i
        For i = 1 To 4
            For j = 1 To 4
                b1 = 0
                For k = 1 To 4
                    b1 = b1 + buf1(i, k) * buf4(k, j)
                Next k
                trbuf(i, j) = b1
            Next j
        Next i

    End Sub

    Sub UFileAdd(ByVal k As Integer)
        'UFileの追加処理
        'k･･･ﾃﾞｰﾀ種類番号
        Dim i, ic, j As Integer
        ic = 0
        For i = 2 To 6
            If UFile(k, i) = UFile(k, 0) Then
                For j = i - 1 To 0 Step -1
                    UFile(k, j + 1) = UFile(k, j)
                Next j
                ic = 1
                Exit For
            End If
        Next i
        If ic = 0 Then
            For i = 5 To 0 Step -1
                UFile(k, i + 1) = UFile(k, i)
            Next i
        End If
        UseFileOut()       'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し

    End Sub

    Sub UseFileIn()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ読み込み
        Dim ftmp$
        Dim i, k As Integer

        ftmp$ = DirOpen$ + "\UseFile.tmp"
        Try
            Dim sr As New IO.StreamReader(ftmp$)
            For k = 1 To 5
                For i = 1 To 6
                    UFile(k, i) = sr.ReadLine
                Next i
            Next k
            sr.Close()
        Catch
            '処理不要
        End Try

    End Sub

    Sub UseFileOut()
        'ﾃﾝﾎﾟﾗﾘｰ･ﾌｧｲﾙ書き出し
        Dim ftmp$
        Dim i, k As Integer

        ftmp$ = DirOpen$ + "\UseFile.tmp"
        Dim sw As New IO.StreamWriter(ftmp$)
        For k = 1 To 5
            For i = 1 To 6
                sw.WriteLine(UFile(k, i))
            Next i
        Next k
        sw.Close()

    End Sub


    Sub XYCalc(ByRef xb As Single, ByRef yb As Single)
        'X,Yの作画座標計算
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            xb = b1 * XYscale + Xlower
            yb = Yupper - b2 * XYscale * Cfix
        Else
            xb = b1 * XYscale / Cfix + Xlower
            yb = Yupper - b2 * XYscale
        End If
    End Sub

    Sub XYCalc2(ByRef icheck As Integer, ByRef xb As Single, _
        ByRef yb As Single)
        'X,Yの作画座標計算
        'icheck･･･作画時のXY入れ替えの有無（0:なし、1:有り）
        'xb･･･X座標値
        'yb･･･Y座標値
        Dim b1, b2 As Single
        b1 = xb - XaxisMin
        b2 = yb - YaxisMin
        If Cfix < 1 Then
            If icheck = 0 Then
                xb = b1 * XYscale + Xlower
                yb = Yupper - b2 * XYscale * Cfix
            Else
                xb = b2 * XYscale + Xlower
                yb = b1 * XYscale * Cfix + Ylower
            End If
        Else
            If icheck = 0 Then
                xb = b1 * XYscale / Cfix + Xlower
                yb = Yupper - b2 * XYscale
            Else
                xb = b2 * XYscale / Cfix + Xlower
                yb = b1 * XYscale + Ylower
            End If
        End If
    End Sub

End Module
